package eu.eiting.StarCharge.control.controller;

import java.awt.Point;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;

import eu.eiting.StarCharge.control.StarCharge;
import eu.eiting.StarCharge.control.states.BaseState;
import eu.eiting.StarCharge.model.structure.Vector2;
import eu.eiting.StarCharge.visualization.menu.container.GUIContainer;
import eu.eiting.StarCharge.visualization.utility.Point2D;
import eu.eiting.StarCharge.visualization.views.GameView;

/**
 * The controller that is used to manage the input while a battle if fought.
 * 
 * @author Caspar Eiting
 *
 */
public class GameController extends BaseController {
	// CLASS ATTRIBUTES
	// ============================================
	private static final int ACCELERATION_KEY 	= KeyEvent.VK_W;
	private static final int DECELERATION_KEY 	= KeyEvent.VK_S;
	private static final int STRAFE_LEFT_KEY 	= KeyEvent.VK_A;
	private static final int STRAFE_RIGHT_KEY	= KeyEvent.VK_D;
	private static final int ATTACK_KEY			= KeyEvent.VK_SPACE;
	// ============================================
	
	// OBJECT ATTRIBUTES
	// ============================================
	private Point mouseTarget					= Point2D.ZERO;
	private boolean accelerationKey				= false;
	private boolean decelerationKey				= false;
	private boolean strafeLeftKey				= false;
	private boolean strafeRightKey				= false;
	private boolean attackKey					= false;
	private GUIContainer gui					= null;
	// ============================================

	// CONSTRCUTOR
	// ============================================
	public GameController(GUIContainer gui){
		super();
		this.gui		= gui;
	}
	// ============================================
	
	// OBJECT METHODS
	// ============================================
	public void update(){
		gui.currentMenu.update();
		
		if(getGameModel() != null && getGameModel().player.getStarShip() != null){
			if(accelerationKey){
				getGameModel().player.getStarShip().forward(true);
			} else {
				getGameModel().player.getStarShip().forward(false);
			}

			if(decelerationKey){
				getGameModel().player.getStarShip().backward(true);
			} else {
				getGameModel().player.getStarShip().backward(false);
			}

			if(strafeLeftKey){
				getGameModel().player.getStarShip().left(true);
			} else {
				getGameModel().player.getStarShip().left(false);
			}

			if(strafeRightKey){
				getGameModel().player.getStarShip().right(true);
			} else {
				getGameModel().player.getStarShip().right(false);
			}

			if(attackKey){
				getGameModel().player.getStarShip().WEAPONS.fire();
			}

			getGameModel().player.getStarShip().setRotationTarget(Vector2.substract( GameView.getScreen().screenToWorld(mouseTarget), getGameModel().player.getStarShip().getTransform().getPosition()));
		}
	}
	
	@Override
	public void keyPressed(KeyEvent e) {
		if (StarCharge.getCurrentStateType() == BaseState.States.GAME_STATE) {
			processKeysPressedIngame(e);
		} else {
			processKeysPressedOutgame(e);
		}
	}

	@Override
	public void keyReleased(KeyEvent e) {
		if (StarCharge.getCurrentStateType() == BaseState.States.GAME_STATE) {
			processKeysReleasedIngame(e);
		} else {
			dropKeys();
		}
	}

	@Override
	public void keyTyped(KeyEvent arg0) {}
	
	@Override
	public void mouseDragged(MouseEvent e) {
		if (StarCharge.getCurrentStateType() == BaseState.States.GAME_STATE) {
			attackKey = true;
			mouseTarget = e.getPoint();
		}
	}

	@Override
	public void mouseMoved(MouseEvent e) {
		if (StarCharge.getCurrentStateType() == BaseState.States.GAME_STATE) {
			mouseTarget = e.getPoint();
		}

		if (gui.currentMenu.isActive()) {
			gui.currentMenu.onMouseMotion(e);
		}
	}

	@Override
	public void mouseClicked(MouseEvent e) {
		if (gui.currentMenu.isActive()) {
			gui.currentMenu.onMousePress(e);
		}
	}

	@Override
	public void mouseEntered(MouseEvent arg0) {}

	@Override
	public void mouseExited(MouseEvent arg0) {}

	@Override
	public void mousePressed(MouseEvent arg0) {
		if (StarCharge.getCurrentStateType() == BaseState.States.GAME_STATE) {
			attackKey = true;
		}
	}

	@Override
	public void mouseReleased(MouseEvent e) {
		if (StarCharge.getCurrentStateType() == BaseState.States.GAME_STATE) {
			attackKey = false;
		}
	}
	
	private void selectSlot(int slot){
		if(getGameModel().player.getStarShip().WEAPONS.getSlotCount() > slot){
			for (int index = 0; index < getGameModel().player.getStarShip().WEAPONS.getSlotCount(); index++) {
				getGameModel().player.getStarShip().WEAPONS.getSlot(index).setActive(false);
			}
			
			getGameModel().player.getStarShip().WEAPONS.getSlot(slot).setActive(true);
		}
	}
	
	private void processKeysPressedIngame(KeyEvent e){
		switch (e.getKeyCode()) {
		case KeyEvent.VK_1:
			selectSlot(0);
			break;
		case KeyEvent.VK_2:
			selectSlot(1);
			break;
		case KeyEvent.VK_3:
			selectSlot(2);
			break;
		case KeyEvent.VK_4:
			selectSlot(3);
			break;
		case ACCELERATION_KEY:
			accelerationKey = true;
			break;
		case DECELERATION_KEY:
			decelerationKey = true;
			break;
		case STRAFE_RIGHT_KEY:
			strafeRightKey = true;
			break;
		case STRAFE_LEFT_KEY:
			strafeLeftKey = true;
			break;
		case ATTACK_KEY:
			attackKey = true;
			break;
		case KeyEvent.VK_ESCAPE:
			StarCharge.changeState(BaseState.States.PAUSE_STATE);
			gui.changeMenu(GUIContainer.Menus.GAME_MENU);
			break;
		case KeyEvent.VK_UP:
			StarCharge.changeState(BaseState.States.MENU_STATE);
			break;
		case KeyEvent.VK_DOWN:
			StarCharge.changeState(BaseState.States.GAME_STATE);
			break;
		default:
			break;
		}
	}
	
	private void processKeysPressedOutgame(KeyEvent e){
		switch (e.getKeyCode()) {
		case KeyEvent.VK_ESCAPE:
			StarCharge.changeState(BaseState.States.GAME_STATE);
			gui.changeMenu(GUIContainer.Menus.GAME_HUD);
			break;
		default:
			break;
		}
	}
	
	private void processKeysReleasedIngame(KeyEvent e){
		switch (e.getKeyCode()) {
		case ACCELERATION_KEY:
			accelerationKey = false;
			break;
		case DECELERATION_KEY:
			decelerationKey = false;
			break;
		case STRAFE_RIGHT_KEY:
			strafeRightKey = false;
			break;
		case STRAFE_LEFT_KEY:
			strafeLeftKey = false;
			break;
		case ATTACK_KEY:
			attackKey = false;
			break;
		default:
			break;
		}
	}
	
	private void dropKeys(){
		accelerationKey				= false;
		decelerationKey				= false;
		strafeLeftKey				= false;
		strafeRightKey				= false;
		attackKey					= false;
	}
	// ============================================
}
