package eu.eiting.StarCharge.model;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Set;

import eu.eiting.StarCharge.model.listener.SolarSystemListener;
import eu.eiting.StarCharge.model.objects.Projectile;
import eu.eiting.StarCharge.model.solarSystem.Fleet;
import eu.eiting.StarCharge.model.solarSystem.Location;
import eu.eiting.StarCharge.model.solarSystem.SolarSystem;
import eu.eiting.StarCharge.model.structure.Vector2;

public class GameModel implements Serializable {
	// CLASS ATTRIBUTES
	//====================================
	private static final long serialVersionUID								= -7244498758844521225L;
	private static GameModel current										= null;
	private static final Set<SolarSystemListener> fleetChangeListeners		= new HashSet<>();
	private static final Set<SolarSystemListener> locationChangeListeners	= new HashSet<>();
	//====================================
	
	// OBJECT ATTRIBUTES
	//====================================
	public final HumanPlayer player								= new HumanPlayer();
	public final ArrayList<AIPlayer> players					= new ArrayList<>();
	public SolarSystem solarSystem								= new SolarSystem();
	private BattleField battleField								= null;
	public final Vector2 startPosition							= new Vector2(320, 400);
	//====================================
	
	// CONSTRUCTOR
	//====================================
	private GameModel(){
		players.add(new AIPlayer("A Name"));
		
		current = this;
	}
	//====================================

	// CLASS METHODS
	//====================================
	public static GameModel New(){
		return new GameModel();
	}
	
	public static GameModel Load(String filepath){
		ObjectInputStream objInStream = null;
		
		try {
			objInStream = new ObjectInputStream(new FileInputStream(filepath));
		
			GameModel loadedModel = (GameModel) objInStream.readObject();
		
			current = loadedModel;
			
			return loadedModel;
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		} finally {
			try {
				objInStream.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}

		return new GameModel();
	}
	
	public static void save(String filepath){
		ObjectOutputStream objOutStream = null;
		
		try {
			objOutStream = new ObjectOutputStream(new FileOutputStream(filepath));
		
			objOutStream.writeObject(current);
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			try {
				objOutStream.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}
	
	public static void addProjectile(Projectile projectile){
		current.battleField.getProjectiles().add(projectile);
	}
		
	public static void removeProjectile(Projectile projectile){
		current.battleField.getProjectiles().remove(projectile);
	}
	
	public static void raiseOnFleetAdditionEvent(Fleet fleet){
		for (SolarSystemListener listener : fleetChangeListeners) {
			listener.onFleetAddition(fleet);
		}
	}
	
	public static void raiseOnFleetRemovalEvent(Fleet fleet){
		for (SolarSystemListener listener : fleetChangeListeners) {
			listener.onFleetRemoval(fleet);
		}
	}
	
	public static void addLocation(Location location){
		current.solarSystem.addLocation(location);
		
		for (SolarSystemListener listener : locationChangeListeners) {
			listener.onLocationAddition(location);
		}
	}
	
	public static void removeLocation(Location location){
		current.solarSystem.removeFleet(location);
		
		for (SolarSystemListener listener : locationChangeListeners) {
			listener.onLocationRemoval(location);
		}
	}
	
	public static void addFleetChangeListener(SolarSystemListener listener){
		fleetChangeListeners.add(listener);
	}
	
	public static void removeFleetChangeListener(SolarSystemListener listener){
		fleetChangeListeners.remove(listener);
	}
	
	public static void addLocationChangeListener(SolarSystemListener listener){
		locationChangeListeners.add(listener);
	}
	
	public static void removeLocationChangeListener(SolarSystemListener listener){
		locationChangeListeners.remove(listener);
	}
	//====================================
	
	// OBJECT METHODS
	//====================================
	public void prepareBattleField(Fleet aggressor, Fleet defender){
		this.battleField = new BattleField(aggressor, defender);
	}
	
	public void terminateBattleField(){
		this.battleField = null;
	}
	
	public boolean activeBattleField(){
		if (battleField != null) {
			return true;
		} else {
			return false;
		}
	}
	//====================================
	
	// GETTER AND SETTER
	//====================================
	public BattleField getBattleField(){
		return battleField;
	}
	//====================================
}