package eu.eiting.StarCharge.model.objects.starship;

import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
import java.io.IOException;

import eu.eiting.StarCharge.model.Drawable;
import eu.eiting.StarCharge.model.Updateable;
import eu.eiting.StarCharge.model.objects.MotionObject;
import eu.eiting.StarCharge.model.objects.starship.slots.WeaponSlot;
import eu.eiting.StarCharge.model.objects.starship.systems.coreSystem.Engine;
import eu.eiting.StarCharge.model.objects.starship.systems.coreSystem.Hull;
import eu.eiting.StarCharge.model.objects.starship.systems.coreSystem.Shield;
import eu.eiting.StarCharge.model.objects.starship.systems.coreSystem.Thruster;
import eu.eiting.StarCharge.model.objects.starship.systems.coreSystem.WeaponSystem;
import eu.eiting.StarCharge.model.objects.starship.systems.weaponSystem.Pulsorbeam;
import eu.eiting.StarCharge.model.resources.Textures;
import eu.eiting.StarCharge.model.resources.datatypes.BufferedImageProxy;
import eu.eiting.StarCharge.model.structure.Vector2;

/**
 * The StarShip represents all attributes and methods of a ship within the game
 * 
 * @author The Singularity
 *
 */
public class StarShip extends MotionObject implements Updateable, Drawable{
	// CLASS ATTRIBUTES
	//====================================
	private static final long serialVersionUID = -5740907117248125167L;
	//====================================
	
	// OBECT ATTRIBUTES
	//====================================
	protected int width								= 0;
	protected int length							= 0;
	public final Shield SHIELD;
	public final Hull HULL;
	public final Engine ENGINE;
	public final WeaponSystem WEAPONS;
	private Vector2 rotationTarget					= null;
	private double rotationMaximum					= 0.0;
	private transient BufferedImageProxy apperance	= null;
	private String apperanceKey						= "1";
	//====================================

	// CONSTRUCTOR
	//====================================
	public StarShip(Vector2 position, int weaponSlots, String type){
		this(position, 32, 32, weaponSlots, type);
	}
	
	public StarShip(Vector2 position, int width, int length, int weaponSlots, String type){
		this.velocity			= Vector2.ZERO;
		this.rotationTarget 	= Vector2.FORWARD;
		this.width				= width;
		this.length				= length;
		this.rotationMaximum	= 5;
		this.apperanceKey		= type;
		
		this.getTransform().setPosition(position);
		
		// TODO: RMEOVE ... load from xml or file
		// set up the systems of the StarShip
		//====================================
		// TODO: load configurations for each system from file
		this.HULL		= new Hull(Vector2.ZERO, 3, 125, 2);
		this.SHIELD		= new Shield(Vector2.ZERO, 2, 100, 0.25f);
		this.ENGINE		= new Engine(Vector2.ZERO, 1, 0.1f);
		this.WEAPONS	= new WeaponSystem(1, 3);
		// TODO: remove
		WeaponSlot slot = new WeaponSlot(Vector2.ZERO, 0);
		slot.getTransform().setParent(getTransform());
		slot.getTransform().setPosition(new Vector2(-20, -20));
		WEAPONS.addSlot(slot);
		
		slot = new WeaponSlot(Vector2.ZERO, 0);
		slot.getTransform().setParent(getTransform());
		slot.getTransform().setPosition(new Vector2(20, -20));
		WEAPONS.addSlot(slot);

		this.WEAPONS.addSlot(Vector2.ZERO, 2);
		this.WEAPONS.getSlot(2).getTransform().setParent(getTransform());
		
		this.WEAPONS.addWeapon(0, new Pulsorbeam(this, WEAPONS.getSlot(0)));
		this.WEAPONS.addWeapon(1, new Pulsorbeam(this, WEAPONS.getSlot(1)));
		this.WEAPONS.addWeapon(2, new Pulsorbeam(this, WEAPONS.getSlot(2)));
		//====================================

		this.apperance = Textures.getResource(BufferedImageProxy.class, apperanceKey);
	}
	//====================================
	
	// OBJECT METHODS
	//====================================
	@Override
	public void update() {
		updateDirection();

		WEAPONS.update();
		SHIELD.update();
		HULL.update();
		
		updateMotion();
	}
	
	private void updateDirection(){
		double angle = Vector2.getAngle(getTransform().getDirection(), rotationTarget);
		
		if(angle > rotationMaximum) {
			getTransform().rotate(rotationMaximum);
		} else if(angle < -rotationMaximum){
			getTransform().rotate(-rotationMaximum);
		} else {
			getTransform().rotate(angle);
		}
	}

	private void updateMotion() {
		Vector2 acceleration = ENGINE.getAcceleration(getTransform().getDirection());

		if(acceleration.equals(Vector2.ZERO) && velocity.magnitude > 0.0f){
			acceleration = Vector2.invert(velocity);
			
			if (acceleration.magnitude > 0.025f) {
				acceleration = new Vector2(acceleration, 0.025f);
			}
		}
		
		velocity  = Vector2.add(velocity, acceleration);
		
		if(velocity.magnitude > 10.0f){
			velocity = new Vector2(velocity, 10.0f);
		}
		
		getTransform().setPosition(Vector2.add(getTransform().getPosition(), velocity));
	}
	
	public void left(boolean value){
		ENGINE.right(value);
	}
	
	public void right(boolean value){
		ENGINE.left(value);
	}

	public void forward(boolean value){
		ENGINE.forward(value);
	}
	
	public void backward(boolean value){
		ENGINE.backward(value);
	}
	
	private void writeObject(java.io.ObjectOutputStream out) throws IOException{
		out.defaultWriteObject();
	}
	
	private void readObject(java.io.ObjectInputStream in) throws IOException, ClassNotFoundException{
		 in.defaultReadObject();
		 
		 this.apperance = Textures.getResource(BufferedImageProxy.class, apperanceKey);
	}
	//====================================
	
	// GETTER AND SETTER
	//====================================
	public void setRotationTarget(Vector2 direction) {
		rotationTarget = direction;
	}
	
	@Override
	public BufferedImage getImage() {
		BufferedImage image = new BufferedImage((int) (apperance.getSource().getWidth() * 1.5), (int) (apperance.getSource().getHeight() * 1.5), BufferedImage.TYPE_INT_ARGB);

		BufferedImage ship = new BufferedImage(apperance.getSource().getWidth(), apperance.getSource().getHeight(), BufferedImage.TYPE_INT_ARGB);
		
		Graphics2D shipG2D = (Graphics2D) ship.getGraphics();
		
		
		for (Thruster item : ENGINE.THRUSTERS) {
			
			if(item.getActive()){
				BufferedImage flame = item.getImage().getSource();
				
				shipG2D.drawImage(flame, (int) item.POSITION.x - flame.getWidth() / 2, (int) item.POSITION.y - flame.getHeight() / 2, null);
			}
		}
		
		shipG2D.drawImage(apperance.getSource(), 0, 0, null);

		image = new BufferedImage((int) (ship.getWidth() * 1.5), (int) (ship.getHeight() * 1.5), BufferedImage.TYPE_INT_ARGB);

		Graphics2D g2d = (Graphics2D) image.getGraphics();

		g2d.rotate(Math.toRadians(getTransform().getRotation()), (int) (image.getWidth() / 2), (int) (image.getHeight() / 2));

		g2d.drawImage(ship, (int) (ship.getWidth() / 4), (int) (ship.getHeight() / 4), null);

		return image;
	}

	public int getWidth() {
		return width;
	}

	public int getLength() {
		return length;
	}
	
	
	//====================================
}
