package eu.eiting.StarCharge.model.objects.starship.slots;

import java.awt.image.BufferedImage;

import eu.eiting.StarCharge.model.Drawable;
import eu.eiting.StarCharge.model.Updateable;
import eu.eiting.StarCharge.model.Usable;
import eu.eiting.StarCharge.model.objects.GameObject;
import eu.eiting.StarCharge.model.objects.starship.systems.weaponSystem.BaseWeapon;
import eu.eiting.StarCharge.model.resources.ResourceArchive;
import eu.eiting.StarCharge.model.structure.Vector2;

/**
 * The representation of a slot of the weapon system.
 * 
 *  - each slot can contain a single weapon
 *  - the position is used to determine from where the projectil is launched 
 * 
 * @author Caspar Eiting
 *
 */
public class WeaponSlot extends GameObject implements Updateable, Drawable, Usable {
	// CLASS ATTRIBUTES
	//====================================
	private static final long serialVersionUID = 1L;
	//====================================
	
	// OBJECT ATTRIBUTES
	//====================================
	private BaseWeapon installedSystem		= null;
	private int levelRequirement			= 0;
	private boolean enabled					= false;
	private boolean isActive				= false;
	//====================================
	
	// CONTRUCTOR
	//====================================
	public WeaponSlot(Vector2 position, int levelRequirement){
		this.levelRequirement				= levelRequirement;
	}
	//====================================

	// OBJECT METHODS
	//====================================
	@Override
	public void update() {
		if(enabled){
			installedSystem.update();
		}
	}
	
	public void addWeapon(BaseWeapon weapon){
		if(enabled){
			installedSystem = weapon;
		}
	}

	public BaseWeapon removeWeapon() {
		if (installedSystem != null) {
			BaseWeapon removed = installedSystem;

			installedSystem = null;

			return removed;
		} else {
			return null;
		}
	}
	
	@Override
	public void use() {
		if (enabled) {
			installedSystem.fire();
		}
	}
	//====================================
	
	// GETTER AND SETTER
	//====================================
	@Override
	public BufferedImage getImage() {
		return null;
	}
	
	public int getLevelRequirement(){
		return levelRequirement;
	}
	
	public void setLevelRequirement(int level){
		this.levelRequirement = level;
	}
	
	public void isEnabled(boolean enabled){
		this.enabled = enabled;
	}
	
	public boolean getEnabled(){
		return enabled;
	}
	
	public String getWeaponType(){
		if(enabled){
			if(installedSystem != null){
				return installedSystem.getType();
			} else {
				return ResourceArchive.KEY_EMPTY_ELEMENT;
			}
		} else {
			return ResourceArchive.KEY_DISABLED_ELEMENT;
		}
	}
	
	public BaseWeapon getWeapon(){
		return installedSystem;
	}
	
	public boolean isActive() {
		return isActive;
	}
	
	public void setActive(boolean isActive) {
		this.isActive = isActive;
	}
	//====================================
}
