package eu.eiting.StarCharge.model.objects.starship.systems.coreSystem;

import java.io.IOException;
import java.io.Serializable;

import eu.eiting.StarCharge.model.resources.Textures;
import eu.eiting.StarCharge.model.resources.datatypes.ImageCircle;
import eu.eiting.StarCharge.model.resources.datatypes.BufferedImageProxy;
import eu.eiting.StarCharge.model.structure.Vector2;

/**
 * Representation of a single thruster that is used by an engine.
 * 
 *  - contains the the power of the thruster and its direction
 *  
 * @author Caspar Eiting
 *
 */
public class Thruster implements Serializable {
	// CLASS ATTRIBTES
	//====================================
	private static final long serialVersionUID = 7556550524587500759L;
	//====================================

	// OBJECT ATTRIBUTES
	//====================================
	private boolean active							= false;
	private double radian							= 0.0f;
	private float power								= 1.0f;
	private transient BufferedImageProxy removeTemp = null;
	private String apperanceID						= "engine";
	public final Vector2 POSITION;
	//====================================

	// CONSTRUCTOR
	//====================================
	public Thruster(Vector2 position, double angle, float power) {
		this.POSITION	= position;
		this.radian		= Math.toRadians(angle);
		this.power		= power;
		this.removeTemp = Textures.getResource(ImageCircle.class, apperanceID).getImage((int) Math.toDegrees(radian));
	}
	//====================================

	// OBJECT METHODS
	//====================================
	public void isActivate(boolean state) {
		this.active = state;
	}

	public boolean getActive() {
		return active;
	}

	//====================================

	// GETTER AND SETTER
	//====================================
	public BufferedImageProxy getImage() {
		return removeTemp;
	}

	public Vector2 getAcceleration(Vector2 shipDirection) {
		Vector2 thrust = Vector2.ZERO;

		if (active == true) {
			thrust = Vector2.normalize(shipDirection);

			thrust = Vector2.rotate(thrust, radian);

			thrust = Vector2.multiply(thrust, power);
		}

		return thrust;
	}

	public float getPower() {
		return power;
	}

	public void setPower(float power) {
		this.power = power;
	}

	public double getAngle() {
		return Math.toDegrees(radian);
	}

	public void setAngle(double angle) {
		radian = Math.toRadians(angle);
	}

	private void writeObject(java.io.ObjectOutputStream out) throws IOException {
		out.defaultWriteObject();
	}

	private void readObject(java.io.ObjectInputStream in) throws IOException, ClassNotFoundException {
		in.defaultReadObject();

		removeTemp = Textures.getResource(ImageCircle.class, apperanceID).getImage((int) Math.toDegrees(radian));
	}
	//====================================
}
