package eu.eiting.StarCharge.model.objects.starship.systems.coreSystem;

import java.util.ArrayList;

import eu.eiting.StarCharge.model.Levelable;
import eu.eiting.StarCharge.model.objects.starship.slots.WeaponSlot;
import eu.eiting.StarCharge.model.objects.starship.systems.ShipSystem;
import eu.eiting.StarCharge.model.objects.starship.systems.weaponSystem.BaseWeapon;
import eu.eiting.StarCharge.model.structure.Vector2;

/**
 * The manages of weapon slots and the installed weapons.
 *  
 * @author Caspar Eiting
 *
 */
public class WeaponSystem extends ShipSystem implements Levelable{
	// CLASS ATTRIBUTES
	//====================================
	private static final long serialVersionUID = 8692498988591303773L;
	//====================================
	
	// OBJECT ATTRIBUTES
	//====================================
	private ArrayList<WeaponSlot> slots = null;
	private int level					= 0;
	private int maxLevel 				= 0;
	//====================================
	
	// CONSTRUCTOR
	//====================================
	public WeaponSystem(int level, int maxLevel) {
		this.maxLevel	= maxLevel;
		
		slots			= new ArrayList<WeaponSlot>();
		
		setLevel(level);
	}
	//====================================

	// OBJECT METHODS
	//====================================
	@Override
	public void update() {
		for (WeaponSlot slot : slots) {
			if (slot.getEnabled() == true) {
				slot.update();	
			}
		}
	}
	
	public void addSlot(Vector2 position, int levelRequirement){
		addSlot(new WeaponSlot(position, levelRequirement));
	}

	public void addSlot(WeaponSlot slot){
		slots.add(slot);
		
		if (slot.getLevelRequirement() <= this.level) {
			slot.isEnabled(true);
		} else {
			slot.isEnabled(false);
		}
		
		if(slots.size() == 1){
			slots.get(0).setActive(true);
		}
	}
	
	public BaseWeapon removeSlot(int index){
		if (slots.get(index).getWeapon() != null) {
			BaseWeapon weaponInSlot = slots.get(index).getWeapon();
			
			slots.remove(index);
			
			return weaponInSlot;
		} else {
			slots.remove(index);
			
			return null;
		}
	}
	
	public void addWeapon(int slot, BaseWeapon weapon){
		if(slot >= 0 && slot < slots.size()){
			slots.get(slot).addWeapon(weapon);
		}
	}
	
	public void fire(){
		for (WeaponSlot slot : slots) {
			if(slot.isActive()){
				slot.use();
			}
		}
	}
	//====================================

	// GETTER AND SETTER
	//====================================
	@Override
	public void setLevel(int level) {
		if (level < maxLevel && level >= 0) {
			this.level = level;
		}
		
		for (WeaponSlot slot : slots) {
			if (slot.getLevelRequirement() <= this.level) {
				slot.isEnabled(true);
			} else {
				slot.isEnabled(false);
			}
		}
	}

	@Override
	public int getLevel() {
		return level;
	}
	
	public WeaponSlot getSlot(int index){
		if(slots.size() > index){
			return slots.get(index);
		}
		
		return null;
	}
	
	public int getSlotCount(){
		return slots.size();
	}
	//====================================
}
