package eu.eiting.StarCharge.model.resources;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.WeakHashMap;

import javax.imageio.ImageIO;

public abstract class ImageTool {
	// CLASS ATTRIBUTES
	// ====================================
	public static final BufferedImage MISSING;
	private static final WeakHashMap<String, BufferedImage> LOADED_IMAGES = new WeakHashMap<>();
	// ====================================

	// CLASS INITIALIZER
	// ====================================
	static {
		// prepare image that is used for elements
		// that are not loaded or couldn't be found
		// ====================================
		MISSING = new BufferedImage(16, 16, BufferedImage.TYPE_INT_ARGB);
		
		Graphics g = MISSING.getGraphics();
		
		g.setColor(Color.BLACK);
		g.fillRect(0, 0, MISSING.getWidth(), MISSING.getHeight());
		
		g.setColor(Color.MAGENTA);
		g.fillRect(0, 0, MISSING.getWidth() / 2, MISSING.getHeight() / 2);
		g.fillRect(MISSING.getWidth() / 2, MISSING.getHeight() / 2, MISSING.getWidth(), MISSING.getHeight());
		// ====================================
	}
	// ====================================

	// CLASS METHODS
	// ====================================
	/**
	 * This method loads a single image from the filename.
	 * 
	 * @param filename Path to the file
	 * @return The loaded image or a blank image filled with the BLANK_COLOR
	 * @throws IOException The exception that shows that there was a problem with the file
	 */
	public static BufferedImage load(String filename) {
		if(LOADED_IMAGES.containsKey(filename) && LOADED_IMAGES.get(filename) != null){
			return LOADED_IMAGES.get(filename);
		} else {
			BufferedImage image = null;
			
			try {
				image = ImageIO.read(new File(filename));
			} catch (IOException e) {
				image = MISSING;
			}
			
			LOADED_IMAGES.put(filename, image);
			
			return image;
		}
	}
	// ====================================
}
