package eu.eiting.StarCharge.model.resources;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import java.util.WeakHashMap;

import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamReader;

import eu.eiting.StarCharge.model.resources.infos.ImageListInfo;
import eu.eiting.StarCharge.model.resources.infos.ImageCircleInfo;
import eu.eiting.StarCharge.model.resources.infos.ImageInfo;
import eu.eiting.StarCharge.model.resources.infos.ResourceInfo;

public class ResourceArchive {
	// OBJECT ATTRIBUTES
	// ============================================
	private final Map<String, ResourceInfo> RESOURCE_INFOS	= new HashMap<>();
	private final Map<String, Object> RESOURCES				= new WeakHashMap<>();
	@SuppressWarnings("rawtypes")
	private final Map<Class, Object> DEFAULT_RESOURCES		= new HashMap<>();
	// ============================================
	
	// CONSTRCUTORS
	// ============================================
	public ResourceArchive(XMLStreamReader reader){
		int counter					= 0;
		XmlBufferedReader stream	= new XmlBufferedReader(reader);

		try {
			while(stream.hasElement("Image")){
				ImageInfo info = new ImageInfo(reader.getAttributeValue(0), reader.getAttributeValue(1));

				AddRessourceInfo(info);
				
				counter++;
			}
			
			counter = 0;
			
			while(stream.hasElement("ImageArray")){
				String key				= reader.getAttributeValue(0);
				ArrayList<String> files = new ArrayList<>();

				while (stream.hasElement("Image")) {
					files.add(reader.getAttributeValue(0));
				}
				
				AddRessourceInfo(new ImageListInfo(key, files));
				
				counter ++;
			}
	
			System.out.println("ImageArrays loaded: " + counter);
			
			counter = 0;
			
			while(stream.hasElement("ImageCircle")){
				ImageCircleInfo info = new ImageCircleInfo(reader.getAttributeValue(0), reader.getAttributeValue(2), Integer.parseInt(reader.getAttributeValue(1)));
				
				AddRessourceInfo(info);
				
				counter++;
			}
			
			System.out.println("ImageCircle loaded: " + counter);
		} catch (XMLStreamException e) {
			e.printStackTrace();
		}
	}
	// ============================================
	
	// GETTER AND SETTER
	// ============================================
	private void AddRessourceInfo(ResourceInfo info){
		if(!RESOURCE_INFOS.containsKey(info.getID())){
			RESOURCE_INFOS.put(info.getID(), info);
		} else {
			throw new RuntimeException("Duplicate KEY[" + info.getID() + "] in RessourceArchive");
		}
	}

	@SuppressWarnings("unchecked")
	public <T> T getResource(Class<T> classInformation, String key){
		if(RESOURCES.containsKey(key) && RESOURCES.get(key) != null){
			// return resource if it is of the correct type, else log a warning and continue
			if(RESOURCES.get(key).getClass() == classInformation){
				return (T) RESOURCES.get(key);
			} else {
				System.out.println("WARNING: RESOURCE[" + key + "] isn't of CLASS[" + classInformation + "]");
			}
		} else if(RESOURCE_INFOS.containsKey(key)){
			// load resource and try again
			Object resource = RESOURCE_INFOS.get(key).loadRessource();
			
			RESOURCES.put(key, resource);
			
			return getResource(classInformation, key);
		} else {
			// log a warning and return a default resource for the missing resource. Create default resource if necessary
			System.out.println("WARNING: RESOURCE[" + key + "] wasn't found in the archive");
			
			if(DEFAULT_RESOURCES.containsKey(classInformation)){
				return (T) DEFAULT_RESOURCES.get(classInformation);
			} else {
				try {
					T resource = classInformation.newInstance(); 
					
					DEFAULT_RESOURCES.put(classInformation, resource);
					
					return resource;
				} catch (InstantiationException | IllegalAccessException e) {
					e.printStackTrace();
				}
			}
		}

		return null;
	}
	// ============================================

	// TODO: REMOVE!
	public static final String KEY_EMPTY_ELEMENT			= "empty";
	public static final String KEY_DISABLED_ELEMENT			= "disabled";
	
}
