package eu.eiting.StarCharge.model.resources;
import javax.xml.namespace.NamespaceContext;
import javax.xml.namespace.QName;
import javax.xml.stream.Location;
import javax.xml.stream.XMLStreamConstants;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamReader;

/**
 * Proxy for an XMLStreamReader to extend behavior to my liking :)
 * 
 * @author Caspar Eiting
 *
 */
public class XmlBufferedReader implements XMLStreamReader {
	// OBJECT ATTRIBUTES
	// ============================================
	private final XMLStreamReader READER;
	// ============================================
	
	// CONSTRUCTOR
	// ============================================
	public XmlBufferedReader(XMLStreamReader reader){
		this.READER = reader;
	}
	// ============================================
	
	// EXTENSION METHODS
	// ============================================
	public boolean hasElement(String name) throws XMLStreamException{
		boolean isSearching = true;
		
		while(isSearching){
			
			next();
			
			if(getEventType() == XMLStreamConstants.START_ELEMENT && getLocalName().equalsIgnoreCase(name)){
				return true;
			} else if(getEventType() == XMLStreamConstants.END_ELEMENT && !getLocalName().equalsIgnoreCase(name)){
				isSearching = false;
			} else if(getEventType() == XMLStreamConstants.END_DOCUMENT){
				isSearching = false;
			}
		}
		
		return false;
	}
	
	public int nextElement(String name) throws XMLStreamException {
		int result			= 0;
		boolean isSearching = true;
		
		next();
		
		while(isSearching){
			if(getEventType() == XMLStreamConstants.START_ELEMENT && getLocalName().equalsIgnoreCase(name)){
				isSearching = false;
			} else {
				next();
			}
		}
		
		return result;
	}
	// ============================================
	
	// INHERITED METHODS
	// ============================================
	@Override
	public Object getProperty(String name) throws IllegalArgumentException {
		return READER.getProperty(name);
	}

	@Override
	public int next() throws XMLStreamException {
		return READER.next();
	}
	
	

	@Override
	public void require(int type, String namespaceURI, String localName) throws XMLStreamException {
		READER.require(type, namespaceURI, localName);
	}

	@Override
	public String getElementText() throws XMLStreamException {
		return READER.getElementText();
	}

	@Override
	public int nextTag() throws XMLStreamException {
		return READER.nextTag();
	}

	@Override
	public boolean hasNext() throws XMLStreamException {
		return READER.hasNext();
	}

	@Override
	public void close() throws XMLStreamException {
		READER.close();
	}

	@Override
	public String getNamespaceURI(String prefix) {
		return READER.getNamespaceURI(prefix);
	}

	@Override
	public boolean isStartElement() {
		return READER.isStartElement();
	}

	@Override
	public boolean isEndElement() {
		return READER.isEndElement();
	}

	@Override
	public boolean isCharacters() {
		return READER.isCharacters();
	}

	@Override
	public boolean isWhiteSpace() {
		return READER.isWhiteSpace();
	}

	@Override
	public String getAttributeValue(String namespaceURI, String localName) {
		return READER.getAttributeValue(namespaceURI, localName);
	}

	@Override
	public int getAttributeCount() {
		return READER.getAttributeCount();
	}

	@Override
	public QName getAttributeName(int index) {
		return READER.getAttributeName(index);
	}

	@Override
	public String getAttributeNamespace(int index) {
		return READER.getAttributeNamespace(index);
	}

	@Override
	public String getAttributeLocalName(int index) {
		return READER.getAttributeLocalName(index);
	}

	@Override
	public String getAttributePrefix(int index) {
		return READER.getAttributePrefix(index);
	}

	@Override
	public String getAttributeType(int index) {
		return READER.getAttributeType(index);
	}

	@Override
	public String getAttributeValue(int index) {
		return READER.getAttributeValue(index);
	}

	@Override
	public boolean isAttributeSpecified(int index) {
		return READER.isAttributeSpecified(index);
	}

	@Override
	public int getNamespaceCount() {
		return READER.getNamespaceCount();
	}

	@Override
	public String getNamespacePrefix(int index) {
		return READER.getNamespacePrefix(index);
	}

	@Override
	public String getNamespaceURI(int index) {
		return READER.getNamespaceURI(index);
	}

	@Override
	public NamespaceContext getNamespaceContext() {
		return READER.getNamespaceContext();
	}

	@Override
	public int getEventType() {
		return READER.getEventType();
	}

	@Override
	public String getText() {
		return READER.getText();
	}

	@Override
	public char[] getTextCharacters() {
		return READER.getTextCharacters();
	}

	@Override
	public int getTextCharacters(int sourceStart, char[] target, int targetStart, int length) throws XMLStreamException {
		return READER.getTextCharacters(sourceStart, target, targetStart, length);
	}

	@Override
	public int getTextStart() {
		return READER.getTextStart();
	}

	@Override
	public int getTextLength() {
		return READER.getTextLength();
	}

	@Override
	public String getEncoding() {
		return READER.getEncoding();
	}

	@Override
	public boolean hasText() {
		return READER.hasText();
	}

	@Override
	public Location getLocation() {
		return READER.getLocation();
	}

	@Override
	public QName getName() {
		return READER.getName();
	}

	@Override
	public String getLocalName() {
		return READER.getLocalName();
	}

	@Override
	public boolean hasName() {
		return READER.hasName();
	}

	@Override
	public String getNamespaceURI() {
		return READER.getNamespaceURI();
	}

	@Override
	public String getPrefix() {
		return READER.getPrefix();
	}

	@Override
	public String getVersion() {
		return READER.getVersion();
	}

	@Override
	public boolean isStandalone() {
		return READER.isStandalone();
	}

	@Override
	public boolean standaloneSet() {
		return READER.standaloneSet();
	}

	@Override
	public String getCharacterEncodingScheme() {
		return READER.getCharacterEncodingScheme();
	}

	@Override
	public String getPITarget() {
		return READER.getPITarget();
	}

	@Override
	public String getPIData() {
		return READER.getPIData();
	}
	// ============================================
}
