package eu.eiting.StarCharge.model.solarSystem;

import java.util.ArrayList;
import java.util.List;

import eu.eiting.StarCharge.model.objects.GameObject;
import eu.eiting.StarCharge.model.objects.starship.StarShip;
import eu.eiting.StarCharge.model.structure.Vector2;

public class Fleet extends Location {
	// CLASS ATTRIBUTES
	// ====================================
	private static final long serialVersionUID = -6645704596607023175L;
	// ====================================
	
	// OBJECT ATTRIBUTES
	// ====================================
	private GameObject target			= null;
	private final List<StarShip> ships	= new ArrayList<StarShip>();
	private float maxVelocity			= 0.0f;
	private Vector2 velocity			= null;
	// ====================================
	
	// CONSTRUCTOR
	// ====================================
	public Fleet(Vector2 position, String name, String type) {
		super(position, name, type);
		this.maxVelocity	= 1.0f;
	}
	// ====================================

	// OBJECT METHODS
	// ====================================
	public void addShip(StarShip ship){
		ships.add(ship);
	}
	
	public void removeShip(int index){
		ships.remove(index);
	}
	
	public void removeShip(StarShip ship){
		ships.remove(ship);
	}
	
	public void clearShips(){
		ships.clear();
	}
	
	@Override
	public void update() {
		updateMotion();
	}

	private void updateMotion(){
		if (target != null) {
			velocity = Vector2.substract(target.getTransform().getPosition(), getTransform().getPosition());
			
			if (velocity.magnitude > maxVelocity) {
				velocity = new Vector2(velocity, maxVelocity);
			}
			
			getTransform().setPosition(Vector2.add(getTransform().getPosition(), velocity));
		}
	}
	// ====================================

	// GETTER AND SETTER
	// ====================================
	public GameObject getTarget(){
		return target;
	}
	
	public void setTarget(GameObject target){
		this.target = target;
	}
	
	public void setTarget(int x, int y){
		GameObject g = new GameObject();
		g.getTransform().setPosition(new Vector2(x, y));
		this.target = g;
	}
	
	public StarShip getStarShip(int index){
		return ships.get(index);
	}
	
	public int getStarShipCount(){
		return ships.size();
	}
	// ====================================
}
