package eu.eiting.StarCharge.model.solarSystem;

import eu.eiting.StarCharge.model.structure.Vector2;

public class OrbitalSettlement extends Location{
	// CLASS ATTRIBUTES
	// ====================================
	private static final long serialVersionUID = 3337793381800725695L;
	// ====================================
	
	// OBJECT ATTRIBUTES
	// ====================================
	private Star parent					= null;
	private double rotationDegree		= 0.0;
	public final double orbitDistance;
	// ====================================
	
	// CONSTRUCTOR
	// ====================================
	public OrbitalSettlement(double orbitDistance, Star parent, String name, double rotationDegree, double rotationInitial, String type) {
		super(new Vector2((float) parent.getTransform().getPosition().x, (float) (parent.getTransform().getPosition().y - orbitDistance)), name, type);
		
		this.orbitDistance	= orbitDistance;
		this.rotationDegree	= Math.toRadians(rotationDegree);
		this.parent			= parent;
		
		rotate(Math.toRadians(rotationInitial));
	}
	// ====================================
	
	// OBJECT METHODS
	// ====================================
	@Override
	public void update() {
		rotate(rotationDegree);
	}
	
	private void rotate(double radian){
		Vector2 starDistance = Vector2.substract(getTransform().getPosition(), parent.getTransform().getPosition());
		
		starDistance = Vector2.rotate(starDistance, radian);
		
		if (starDistance.magnitude != orbitDistance) {
			starDistance = Vector2.multiply(Vector2.normalize(starDistance), (float) orbitDistance);
		}
		
		getTransform().setPosition(Vector2.add(parent.getTransform().getPosition(), starDistance)); 
	}
	// ====================================
}
