package eu.eiting.StarCharge.model.solarSystem;

import eu.eiting.StarCharge.model.structure.Vector2;

public class Planet extends Star {
	// CLASS ATTRIBUTES
	// ====================================
	private static final long serialVersionUID = 4860007222653302293L;
	// planet types
	public static final String STAR_TYPE_PLANET_E = "p_e";
	public static final String STAR_TYPE_PLANET_F = "p_f";
	public static final String STAR_TYPE_PLANET_G = "p_g";
	public static final String STAR_TYPE_PLANET_J = "p_j";
	public static final String STAR_TYPE_PLANET_L = "p_l";
	public static final String STAR_TYPE_PLANET_M = "p_m";
	public static final String STAR_TYPE_PLANET_N = "p_n";
	// ====================================
	
	// OBJECT ATTRIBUTES
	// ====================================
	private Star parent					= null;
	private double rotationDegree		= 0.0;
	public final double orbitDistance;
	// ====================================
	
	// CONSTRUCTOR
	// ====================================
	public Planet(double orbitDistance, Star parent, String type, double rotationDegree, double rotationInitial) {
		super(new Vector2((float) parent.getTransform().getPosition().x, (float) (parent.getTransform().getPosition().y - orbitDistance)), type);
		
		this.orbitDistance	= orbitDistance;
		this.rotationDegree	= Math.toRadians(rotationDegree);
		this.parent			= parent;
		
		rotate(Math.toRadians(rotationInitial));
	}
	// ====================================
	
	// OBJECT METHODS
	// ====================================
	@Override
	public void update() {
		rotate(rotationDegree);
	}
	
	private void rotate(double radian){
		Vector2 starDistance = Vector2.substract(getTransform().getPosition(), parent.getTransform().getPosition());
		
		starDistance = Vector2.rotate(starDistance, radian);
		
		getTransform().setPosition(Vector2.add(parent.getTransform().getPosition(), starDistance)); 
	}
	// ====================================
}
