package eu.eiting.StarCharge.model.solarSystem;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import eu.eiting.StarCharge.model.Updateable;
import eu.eiting.StarCharge.model.structure.Vector2;

public class SolarSystem implements Updateable, Serializable {
	// CLASS ATTRIBUTES
	// ============================================
	private static final long serialVersionUID = -6752856758418761003L;
	// ============================================
	
	// OBJECT ATTRIBUTES
	// ====================================
	private Star sun												= null;
	private final List<Planet> planets								= new ArrayList<Planet>();
	private final List<Location> locations							= new ArrayList<Location>();
	// ====================================
	
	// CONSTRUCTOR
	// ====================================
	public SolarSystem(){
		// initialize stars
		// ====================================
		this.sun	= new Star(new Vector2(0, 0), Star.STAR_TYPE_SUN_YELLOW);
		
		planets.add(new Planet(64, sun, Planet.STAR_TYPE_PLANET_F, 0.1, 45.0));
		planets.add(new Planet(128, sun, Planet.STAR_TYPE_PLANET_L, 0.01, 135.0));		
		planets.add(new Planet(256, sun, Planet.STAR_TYPE_PLANET_M, 0.01, 323));
		planets.add(new Planet(320, sun, Planet.STAR_TYPE_PLANET_G, 0.01, 270));
		planets.add(new Planet(192, sun, Planet.STAR_TYPE_PLANET_E, 0.01, 96));
		// ====================================
		
		// initialize locations
		// ====================================
		locations.add(new OrbitalSettlement(32, planets.get(0), "1st Fleet", 1.0, 123.0, Location.LOCATION_TYPE_ORBITAL_SETTLEMENT));
		locations.add(new OrbitalSettlement(48, planets.get(3), "1st Fleet", 1.5, 23.0, Location.LOCATION_TYPE_ORBITAL_SETTLEMENT));
		// ====================================
	}
	// ====================================

	// OBJECT METHODS
	// ====================================
	@Override
	public void update() {
		for (Star star : planets) {
			star.update();
		}
		
		for (Location location : locations) {
			location.update();
		}
	}
	// ====================================
	
	// GETTER AND SETTER
	// ====================================
	public Star getSun() {
		return sun;
	}
	
	public List<Location> getLocations() {
		return locations;
	}

	public List<Planet> getPlanets() {
		return planets;
	}

	public void addLocation(Location location) {
		locations.add(location);
	}

	public void removeFleet(Location location) {
		locations.remove(location);
	}
	// ====================================
}
