package eu.eiting.StarCharge.visualization;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.util.HashMap;
import java.util.Map;

import javax.swing.JFrame;

import eu.eiting.StarCharge.control.StarCharge;
import eu.eiting.StarCharge.control.controller.BaseController;
import eu.eiting.StarCharge.control.states.BaseState;
import eu.eiting.StarCharge.model.exceptions.StateNotFoundException;
import eu.eiting.StarCharge.model.listener.ChangeListener;
import eu.eiting.StarCharge.visualization.views.BaseView;
import eu.eiting.StarCharge.visualization.views.GameView;
import eu.eiting.StarCharge.visualization.views.MenuView;
import eu.eiting.StarCharge.visualization.views.SolarSystemView;

public class GameFrame extends JFrame implements ChangeListener<BaseState.States> {
	// CLASS ATTRIBUTES
	// ====================================
	private static final long serialVersionUID = 7857098703212506921L;
	private static Dimension resolution = null;
	// ====================================

	// OBJECT ATTRIBUTES
	// ====================================
	private final Map<BaseState.States, BaseView> views = new HashMap<BaseState.States, BaseView>();
	private BaseState.States activeView = null;
	// ====================================

	// CONSTRUTOR
	// ====================================
	public GameFrame() {
		// retrieve resolution
		resolution = Toolkit.getDefaultToolkit().getScreenSize();

		// initialize the panels of the Frame
		// ====================================
		views.put(BaseState.States.GAME_STATE, new GameView(resolution, "Main_CAMERA"));
		views.put(BaseState.States.MENU_STATE, new MenuView(resolution));
		views.put(BaseState.States.SOLAR_SYSTEM_STATE, new SolarSystemView(resolution));
		// ====================================

		// add the initial panel to the frame
		// ====================================
		try {
			changeView(BaseState.States.MENU_STATE);
		} catch (StateNotFoundException e) {
			e.printStackTrace();
		}
		// ====================================

		// set up the properties of the GameFrame
		// ====================================
		setTitle("Star Charge - Super Space Monkey Invaders from Neptun!");
		setSize(Toolkit.getDefaultToolkit().getScreenSize());
		setResizable(false);
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setUndecorated(true);
		setVisible(true);
		// ====================================

		StarCharge.addGameStateListener(this);
	}

	// ====================================

	// OBJECT METHODS
	// ====================================
	@Override
	public void repaint() {
		for (Component current : getComponents()) {
			current.repaint();
		}
	}

	private boolean changeView(BaseState.States menuState) throws StateNotFoundException {
		BaseView nextView = views.get(menuState);

		if(nextView != null){
			resetView();

			activeView = menuState;

			add(nextView);

			views.get(menuState).requestFocus();

			return true;
		} else {
			throw new StateNotFoundException("GameFrame does not contain a View corresponding to the " + menuState + " GameState!");
		}
		
	}

	private void resetView() {
		for (BaseView view : views.values()) {
			remove(view);
		}
	}
	
	@Override
	public void onChange(BaseState.States obj) {
		if (obj != BaseState.States.PAUSE_STATE) {
			try {
				changeView(obj);
			} catch (StateNotFoundException e) {
				e.printStackTrace();
			}
		}
	}
	// ====================================

	// GETTER AND SETTER
	// ====================================
	public BaseController[] getController() {
		return views.get(activeView).getController();
	}

	public static Dimension getResolution() {
		return resolution;
	}
	// ====================================
}
