package eu.eiting.StarCharge.visualization.menu;

import java.awt.Rectangle;
import java.awt.event.MouseEvent;

import eu.eiting.StarCharge.control.StarCharge;
import eu.eiting.StarCharge.control.states.BaseState;
import eu.eiting.StarCharge.model.resources.Textures;
import eu.eiting.StarCharge.model.resources.datatypes.ImageList;
import eu.eiting.StarCharge.visualization.GameFrame;
import eu.eiting.StarCharge.visualization.menu.container.GUIContainer;
import eu.eiting.StarCharge.visualization.menu.menuElements.ButtonElement;
import eu.eiting.StarCharge.visualization.menu.menuElements.GUIElement;
import eu.eiting.StarCharge.visualization.menu.menuElements.StatusElement;
import eu.eiting.StarCharge.visualization.menu.menuElements.WeaponElement;

public class GameGUI extends BaseGUI {
	// OBJECT ATTRIBUTES
	// ============================================
	private StatusElement STATUS_ELEMENT;
	private ButtonElement FTL_BUTTON;
	private WeaponElement WEAPON_ELEMENT;
	private GUIElement[] ELEMENTS;
	// ============================================
	
	// CONSTRUCTOR
	// ============================================
	public GameGUI(GUIContainer container) {
		super(container);
		
		STATUS_ELEMENT	= new StatusElement(new Rectangle(0, GameFrame.getResolution().height - 64, 512, 64));
		
		ImageList images = Textures.getResource(ImageList.class, "ftl_btn");
		FTL_BUTTON		= new ButtonElement(new Rectangle((GameFrame.getResolution().width - 128) / 2, GameFrame.getResolution().height - 64, 256, 64), images.get(0), images.get(1), images.get(2), images.get(3));

		WEAPON_ELEMENT	= new WeaponElement(new Rectangle(GameFrame.getResolution().width - 256, GameFrame.getResolution().height - 64, 256, 64));

		ELEMENTS	= new GUIElement[3];
		ELEMENTS[0] = STATUS_ELEMENT;
		ELEMENTS[1] = FTL_BUTTON;
		ELEMENTS[2] = WEAPON_ELEMENT;
	}
	// ============================================

	// OBJECT METHODS
	// ============================================
	@Override
	public void update() {
		STATUS_ELEMENT.getHullBar().setMaxLevel(getGameModel().player.getStarShip().HULL.getMaximalHullStrength());
		STATUS_ELEMENT.getHullBar().setPrimaryLevel(getGameModel().player.getStarShip().HULL.getCurrentHullStrength());
		
		STATUS_ELEMENT.getShieldBar().setMaxLevel(getGameModel().player.getStarShip().SHIELD.getMaximalPower());
		STATUS_ELEMENT.getShieldBar().setPrimaryLevel(getGameModel().player.getStarShip().SHIELD.getCurrentPower());
		
		WEAPON_ELEMENT.update();
		
		if(FTL_BUTTON.isActive()){
			getGameModel().player.getFleet().setTarget(null);
			
			StarCharge.changeState(BaseState.States.SOLAR_SYSTEM_STATE);
		}
	}

	@Override
	public void onMousePress(MouseEvent event) {
		for (GUIElement element : ELEMENTS) {
			element.onMousePress(event);
		}
	}

	@Override
	public void onMouseRelease(MouseEvent event) {
		for (GUIElement element : ELEMENTS) {
			element.onMouseRelease(event);
		}
	}

	@Override
	public void onMouseMotion(MouseEvent event) {
		for (GUIElement element : ELEMENTS) {
			element.onMouseMotion(event);
		}
	}
	// ============================================
	
	// GETTER AND SETTER
	// ============================================
	@Override
	public GUIElement[] getElements() {
		return ELEMENTS;
	}
	// ============================================

}
