package eu.eiting.StarCharge.visualization.menu;

import java.awt.Rectangle;
import java.awt.event.MouseEvent;

import eu.eiting.StarCharge.control.StarCharge;
import eu.eiting.StarCharge.control.states.BaseState;
import eu.eiting.StarCharge.model.resources.Textures;
import eu.eiting.StarCharge.model.resources.datatypes.BufferedImageProxy;
import eu.eiting.StarCharge.model.resources.datatypes.ImageList;
import eu.eiting.StarCharge.visualization.GameFrame;
import eu.eiting.StarCharge.visualization.menu.container.GUIContainer;
import eu.eiting.StarCharge.visualization.menu.menuElements.BackgroundElement;
import eu.eiting.StarCharge.visualization.menu.menuElements.ButtonElement;
import eu.eiting.StarCharge.visualization.menu.menuElements.GUIElement;

public class GameMenuGUI extends BaseGUI {
	// OBJECT ATTRIBUTES
	// ====================================
	private final BackgroundElement BACKGROUND;
	private final ButtonElement CONTINUE_BUTTON;
	private final ButtonElement EXIT_BUTTON;
	private final GUIElement[] ELEMENTS;
	// ====================================
	
	// CONSTRUCTOR
	// ====================================
	public GameMenuGUI(GUIContainer container) {
		super(container);
		this.BACKGROUND				= new BackgroundElement(new Rectangle(0, 0, (int) GameFrame.getResolution().getWidth(), (int) GameFrame.getResolution().getHeight()), Textures.getResource(BufferedImageProxy.class, "main_menu_background"));
		
		ImageList images			= Textures.getResource(ImageList.class, "continue_ingame_btn");
		this.CONTINUE_BUTTON		= new ButtonElement(new Rectangle((int) GameFrame.getResolution().getWidth() / 2 - 128, 160, 256, 64), images.get(0), images.get(1), images.get(2), images.get(3));
		
		images						= Textures.getResource(ImageList.class, "exit_ingame_btn");
		this.EXIT_BUTTON			= new ButtonElement(new Rectangle((int) GameFrame.getResolution().getWidth() / 2 - 128, 224, 256, 64), images.get(0), images.get(1), images.get(2), images.get(3));
		
		ELEMENTS	= new GUIElement[3];
		ELEMENTS[0]	= BACKGROUND;
		ELEMENTS[1]	= CONTINUE_BUTTON;
		ELEMENTS[2]	= EXIT_BUTTON;
	}
	// ====================================

	// OBJECT METHODS
	// ====================================
	@Override
	public void update() {}
	
	@Override
	public void onMousePress(MouseEvent event) {
		for (GUIElement element : ELEMENTS) {
			element.onMousePress(event);
		}
		
		if (CONTINUE_BUTTON.isActive()) {
			StarCharge.changeState(BaseState.States.GAME_STATE);

			getParent().changeMenu(GUIContainer.Menus.GAME_HUD);
		} else if (EXIT_BUTTON.isActive()) {
			// TODO: SAVE PROGRESS
			
			getParent().changeMenu(GUIContainer.Menus.GAME_HUD);
			
			StarCharge.changeState(BaseState.States.MENU_STATE);
		}
	}

	@Override
	public void onMouseRelease(MouseEvent event) {
		for (GUIElement element : ELEMENTS) {
			element.onMouseRelease(event);
		}
	}

	@Override
	public void onMouseMotion(MouseEvent event) {
		for (GUIElement element : ELEMENTS) {
			element.onMouseMotion(event);
		}
	}
	// ====================================
	
	// GETTER AND SETTER
	// ====================================
	@Override
	public GUIElement[] getElements() {
		return ELEMENTS;
	}
	// ====================================

	
}
