package eu.eiting.StarCharge.visualization.menu;

import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;

import eu.eiting.StarCharge.control.StarCharge;
import eu.eiting.StarCharge.control.states.BaseState;
import eu.eiting.StarCharge.model.GameModel;
import eu.eiting.StarCharge.model.objects.starship.StarShip;
import eu.eiting.StarCharge.model.profiles.ProfileUtility;
import eu.eiting.StarCharge.model.resources.Textures;
import eu.eiting.StarCharge.model.resources.datatypes.ImageList;
import eu.eiting.StarCharge.model.structure.Vector2;
import eu.eiting.StarCharge.visualization.GameFrame;
import eu.eiting.StarCharge.visualization.menu.container.GUIContainer;
import eu.eiting.StarCharge.visualization.menu.menuElements.BackgroundElement;
import eu.eiting.StarCharge.visualization.menu.menuElements.ButtonElement;
import eu.eiting.StarCharge.visualization.menu.menuElements.GUIElement;

public class MainMenuGUI extends BaseGUI {
	// OBJECT ATTRIBUTES
	// ====================================
	private final ButtonElement START_BUTTON;
	private final ButtonElement CONTINUE_BUTTON;
	private final GUIElement TOP_ELEMENT;
	private final GUIElement BOTTOM_ELEMENT;
	private final ButtonElement BACK_BUTTON;
	private final ButtonElement EXIT_BUTTON;
	private final GUIElement[] ELEMENTS;
	private int elementOffset						= 0;
	// ====================================
	
	// CONSTRUCTOR
	// ====================================
	public MainMenuGUI(GUIContainer container){
		super(container);

		this.elementOffset = (int) (GameFrame.getResolution().getWidth() / 2) - 128;

		ImageList images = Textures.getResource(ImageList.class, "menu_element");	
		TOP_ELEMENT		= new BackgroundElement(new Rectangle(elementOffset, 208, 256, 64), images.get(0));
		BOTTOM_ELEMENT	= new BackgroundElement(new Rectangle(elementOffset, 224, 256, 64), images.get(1));
		
		images			= Textures.getResource(ImageList.class, "new_game_btn");
		START_BUTTON	= new ButtonElement(new Rectangle(elementOffset, 160, 256, 64), images.get(0), images.get(1), images.get(2), images.get(3));
		
		images			= Textures.getResource(ImageList.class, "continue_btn");
		CONTINUE_BUTTON	= new ButtonElement(new Rectangle(elementOffset, 64, 256, 64), images.get(0), images.get(1), images.get(2), images.get(3));
		
		images			= Textures.getResource(ImageList.class, "back_btn");
		BACK_BUTTON		= new ButtonElement(new Rectangle(elementOffset, 240, 256, 64), images.get(0), images.get(1), images.get(2), images.get(3));
		
		images			= Textures.getResource(ImageList.class, "exit_btn");
		EXIT_BUTTON		= new ButtonElement(new Rectangle(elementOffset, 240, 256, 64), images.get(0), images.get(1), images.get(2), images.get(3));

		ELEMENTS	= new GUIElement[5];
		ELEMENTS[0] = TOP_ELEMENT;
		ELEMENTS[1] = START_BUTTON;
		ELEMENTS[2] = BOTTOM_ELEMENT;
		ELEMENTS[3] = BACK_BUTTON;
		ELEMENTS[4] = EXIT_BUTTON;
	}
	// ====================================

	// OBJECT METHODS
	// ====================================
	@Override
	public void update() {
		evaluateSituation();
	}
	
	@Override
	public void onMousePress(MouseEvent event) {
		for (GUIElement element : ELEMENTS) {
			element.onMousePress(event);
		}
		
		if(START_BUTTON.isActive()){
			if(getGameModel() != null){
				getGameModel().player.setStarShip(new StarShip(Vector2.ZERO, 5, "SHIP_type_01"));
			
				evaluateSituation();
				
				StarCharge.changeState(BaseState.States.SOLAR_SYSTEM_STATE);
			}
		}
		
		if(BACK_BUTTON.isActive()){
			getParent().changeMenu(GUIContainer.Menus.PROFILE_MAIN_MENU);
		}
		 
		if(EXIT_BUTTON.isActive()){
			ProfileUtility.saveProfiles();
			
			if(ProfileUtility.getActiveProfil() != null){
				GameModel.save(ProfileUtility.getActiveProfil().getFileName());
			}
			
			System.exit(0);
		}
		
		if(CONTINUE_BUTTON.isActive()){
			StarCharge.changeState(BaseState.States.SOLAR_SYSTEM_STATE);
		}
	}

	@Override
	public void onMouseRelease(MouseEvent event) {
		for (GUIElement element : ELEMENTS) {
			element.onMouseRelease(event);
		}
	}

	@Override
	public void onMouseMotion(MouseEvent event) {
		for (GUIElement element : ELEMENTS) {
			element.onMouseMotion(event);
		}
	}
	
	private void evaluateSituation(){
		if(getGameModel() != null && getGameModel().player.getStarShip() != null){
			TOP_ELEMENT.setPosition(new Point(elementOffset, 112));
			ELEMENTS[1] = CONTINUE_BUTTON;
			START_BUTTON.setEnabled(false);
			CONTINUE_BUTTON.setEnabled(true);
			BOTTOM_ELEMENT.setPosition(new Point(elementOffset, 334));
			BACK_BUTTON.setPosition(new Point(elementOffset, 352));
			EXIT_BUTTON.setPosition(new Point(elementOffset + BACK_BUTTON.getWidth(), 352));
		} else {
			TOP_ELEMENT.setPosition(new Point(elementOffset, 208));
			ELEMENTS[1]	= START_BUTTON;
			CONTINUE_BUTTON.setEnabled(false);
			START_BUTTON.setEnabled(true);
			BOTTOM_ELEMENT.setPosition(new Point(elementOffset, 224));
			BACK_BUTTON.setPosition(new Point(elementOffset, 240));
			EXIT_BUTTON.setPosition(new Point(elementOffset + 128, 240));
		}
	}
	// ====================================
	
	// GETTER AND SETTER
	// ====================================
	@Override
	public GUIElement[] getElements() {
		return ELEMENTS;
	}
	// ====================================
}
