package eu.eiting.StarCharge.visualization.menu;

import java.awt.Rectangle;
import java.awt.event.MouseEvent;
import java.util.ArrayList;

import eu.eiting.StarCharge.model.AIPlayer;
import eu.eiting.StarCharge.model.GameModel;
import eu.eiting.StarCharge.model.listener.SolarSystemListener;
import eu.eiting.StarCharge.model.resources.Textures;
import eu.eiting.StarCharge.model.resources.datatypes.ImageList;
import eu.eiting.StarCharge.model.solarSystem.Fleet;
import eu.eiting.StarCharge.model.solarSystem.Location;
import eu.eiting.StarCharge.model.solarSystem.Planet;
import eu.eiting.StarCharge.visualization.GameFrame;
import eu.eiting.StarCharge.visualization.menu.container.GUIContainer;
import eu.eiting.StarCharge.visualization.menu.menuElements.BackgroundElement;
import eu.eiting.StarCharge.visualization.menu.menuElements.GUIElement;
import eu.eiting.StarCharge.visualization.menu.menuElements.SwitchElement;
import eu.eiting.StarCharge.visualization.menu.menuElements.solarSystem.LocationElement;
import eu.eiting.StarCharge.visualization.menu.menuElements.solarSystem.PlanetElement;

public class SolarSystemGUI extends BaseGUI implements SolarSystemListener {
	// OBJECT ATTRIBUTES
	// ============================================
	private final ArrayList<LocationElement> LOCATIONS	= new ArrayList<>();
	private final ArrayList<PlanetElement> PLANETS		= new ArrayList<>();
	private final ArrayList<LocationElement> FLEETS		= new ArrayList<>();
	private final BackgroundElement CENTER;
	// ============================================

	// CONSTRUCTOR
	// ============================================
	public SolarSystemGUI(GUIContainer container) {
		super(container);

		CENTER =  new BackgroundElement(new Rectangle((int) (GameFrame.getResolution().getWidth() / 2), (int) (GameFrame.getResolution().getHeight() / 2), 0,0), null);
	
		GameModel.addFleetChangeListener(this);
	}
	// ============================================

	// OBJECT METHODS
	// ============================================
	@Override
	public void update() {
		for (LocationElement element : LOCATIONS) {
			element.update();
		}

		for (LocationElement element : FLEETS) {
			element.update();
		}

		for (PlanetElement element : PLANETS) {
			element.update();
		}
	}
	
	@Override
	public void onMousePress(MouseEvent event) {
		for (GUIElement element : FLEETS) {
			element.onMousePress(event);
		}
		
		for (GUIElement element : PLANETS) {
			element.onMousePress(event);
		}
		
		for (GUIElement element : LOCATIONS) {
			element.onMousePress(event);
		}

		for (int index = 0; index < FLEETS.size(); index++) {
			if (FLEETS.get(index).isActive()) {
				if (FLEETS.get(index).getParent() != getGameModel().player.getFleet()) {
					getGameModel().player.getFleet().setTarget(FLEETS.get(index).getParent());
					
					for (SwitchElement element : FLEETS) {
						element.setActive(false);
					}
					
					FLEETS.get(index).setActive(true);
				}
			}
		}
	}

	@Override
	public void onMouseRelease(MouseEvent event) {
		for (GUIElement element : FLEETS) {
			element.onMouseRelease(event);
		}
		
		for (GUIElement element : PLANETS) {
			element.onMouseRelease(event);
		}
		
		for (GUIElement element : LOCATIONS) {
			element.onMouseRelease(event);
		}
	}

	@Override
	public void onMouseMotion(MouseEvent event) {
		for (GUIElement element : FLEETS) {
			element.onMouseMotion(event);
		}
		
		for (GUIElement element : PLANETS) {
			element.onMouseMotion(event);
		}
		
		for (GUIElement element : LOCATIONS) {
			element.onMouseMotion(event);
		}
	}

	private void createPlanets() {
		if (getGameModel() != null) {
			PLANETS.clear();

			for (Planet planet : getGameModel().solarSystem.getPlanets()) {
				ImageList images = Textures.getResource(ImageList.class, planet.TYPE);
				
				PlanetElement element = new PlanetElement(new Rectangle(0, 0, 48, 48), images.get(0), images.get(1), planet);
				
				element.setParent(CENTER);
				
				PLANETS.add(element);
			}
		}
	}

	private void createLocations() {
		if (getGameModel() != null) {
			LOCATIONS.clear();
			
			ImageList images = Textures.getResource(ImageList.class, "orbital_icon");
			
			for (Location location : getGameModel().solarSystem.getLocations()) {
				LocationElement element = new LocationElement(new Rectangle(0, 0, 32, 32), images.get(0), images.get(1), images.get(2), images.get(3), location);
				
				element.setParent(CENTER);
				
				LOCATIONS.add(element);
			}
		}
	}

	private void createFleets() {
		if (getGameModel() != null) {
			FLEETS.clear();
			
			ImageList images = Textures.getResource(ImageList.class, "fleet_icon");
			
			LocationElement element = new LocationElement(new Rectangle(0, 0, 32, 32), images.get(0), images.get(1), images.get(2), images.get(3), getGameModel().player.getFleet());
			
			element.setParent(CENTER);
			
			FLEETS.add(element);
			
			for (AIPlayer player : getGameModel().players) {
				for(int index = 0; index < player.getFleetCount(); index++){
					element = new LocationElement(new Rectangle(0, 0, 32, 32), images.get(0), images.get(1), images.get(2), images.get(3), player.getFleet(0));
					
					element.setParent(CENTER);
					
					FLEETS.add(element);
				}
			}
		}
	}
	

	@Override
	public void onFleetAddition(Fleet fleet) {
		ImageList images = Textures.getResource(ImageList.class, "fleet_icon");
		
		FLEETS.add(new LocationElement(new Rectangle(0, 0, 32, 32), images.get(0), images.get(1), images.get(2), images.get(3), fleet));
	}

	@Override
	public void onFleetRemoval(Fleet fleet) {
		boolean isDone = false;

		for (int i = 0; i < FLEETS.size() && !isDone; i++) {
			if (FLEETS.get(i).getParent() == fleet) {
				FLEETS.remove(i).setParent(null);
				
				FLEETS.remove(i);

				isDone = true;
			}
		}
	}

	@Override
	public void onLocationAddition(Location location) {
		ImageList images = Textures.getResource(ImageList.class, "orbital_icon");
		
		LOCATIONS.add(new LocationElement(new Rectangle(0, 0, 32, 32), images.get(0), images.get(1), images.get(2), images.get(3), location));
	}

	@Override
	public void onLocationRemoval(Location location) {
		boolean isDone = false;

		for (int i = 0; i < LOCATIONS.size() && !isDone; i++) {
			if (LOCATIONS.get(i).getParent() == location) {
				LOCATIONS.remove(i).setParent(null);
				
				LOCATIONS.remove(i);

				isDone = true;
			}
		}
	}

	@Override
	public void onChange(GameModel gameModel) {
		super.onChange(gameModel);

		createPlanets();
		createLocations();
		createFleets();
	}
	// ============================================

	// GETTER AND SETTER
	// ============================================
	@Override
	public GUIElement[] getElements() {
		ArrayList<GUIElement> combinedList = new ArrayList<GUIElement>();

		combinedList.addAll(PLANETS);
		combinedList.addAll(FLEETS);
		combinedList.addAll(LOCATIONS);

		return combinedList.toArray(new GUIElement[combinedList.size()]);
	}
	// ============================================
}
