package eu.eiting.StarCharge.visualization.menu;

import java.awt.Rectangle;
import java.awt.event.MouseEvent;

import eu.eiting.StarCharge.control.StarCharge;
import eu.eiting.StarCharge.control.states.BaseState;
import eu.eiting.StarCharge.model.resources.Textures;
import eu.eiting.StarCharge.model.resources.datatypes.BufferedImageProxy;
import eu.eiting.StarCharge.model.resources.datatypes.ImageList;
import eu.eiting.StarCharge.model.solarSystem.Fleet;
import eu.eiting.StarCharge.model.structure.Vector2;
import eu.eiting.StarCharge.visualization.GameFrame;
import eu.eiting.StarCharge.visualization.menu.container.GUIContainer;
import eu.eiting.StarCharge.visualization.menu.menuElements.ButtonElement;
import eu.eiting.StarCharge.visualization.menu.menuElements.ConfirmDialog;
import eu.eiting.StarCharge.visualization.menu.menuElements.GUIElement;

public class SolarSystemMenuGUI extends BaseGUI {
	// OBJECT ATTRIBUTES
	// ============================================
	private final ButtonElement BACK_BUTTON;
	private ConfirmDialog battleDialog;
	private final double BATTLE_DISTANCE = 10.0;
	// ============================================

	// CONSTRUCTOR
	// ============================================
	public SolarSystemMenuGUI(GUIContainer container) {
		super(container);

		ImageList images = Textures.getResource(ImageList.class, "back_btn");
		
		this.BACK_BUTTON = new ButtonElement(new Rectangle((int) ((GameFrame.getResolution().getWidth() - 128) / 2), (int) GameFrame.getResolution().getHeight() - 64, 128, 64), images.get(0), images.get(1), images.get(2), images.get(3));
	}
	// ============================================

	// OBJECT METHODS
	// ============================================
	@Override
	public void update() {
		updateTarget();
	}

	@Override
	public void onMousePress(MouseEvent event) {
		BACK_BUTTON.onMousePress(event);
		
		if(battleDialog != null){
			battleDialog.onMousePress(event);
			
			if(battleDialog.getValue() == ConfirmDialog.CONFIRM){
				battleDialog = null;
					
				getGameModel().prepareBattleField(getGameModel().player.getFleet(), (Fleet) getGameModel().player.getFleet().getTarget());
					
				getGameModel().player.getFleet().setTarget(null);
				
				BACK_BUTTON.setEnabled(true);
				
				StarCharge.changeState(BaseState.States.GAME_STATE);
			} else if(battleDialog.getValue() == ConfirmDialog.CANCEL){
				getGameModel().player.getFleet().setTarget(null);

				battleDialog = null;
				
				BACK_BUTTON.setEnabled(true);
				
				StarCharge.changeState(BaseState.States.SOLAR_SYSTEM_STATE);
			}
		}
		
		if(BACK_BUTTON.isActive()){
			StarCharge.changeState(BaseState.States.MENU_STATE);
		}
	}

	@Override
	public void onMouseRelease(MouseEvent event) {
		BACK_BUTTON.onMouseRelease(event);
		
		if(battleDialog != null){
			battleDialog.onMouseRelease(event);
		}
	}

	@Override
	public void onMouseMotion(MouseEvent event) {
		BACK_BUTTON.onMouseMotion(event);
		
		if(battleDialog != null){
			battleDialog.onMouseMotion(event);
		}
	}
	
	private void updateTarget() {
		if (getGameModel() != null && getGameModel().player != null && getGameModel().player.getFleet() != null && getGameModel().player.getFleet().getTarget() != null) {
			
			if (Vector2.distance(getGameModel().player.getFleet().getTransform().getPosition(), getGameModel().player.getFleet().getTarget().getTransform().getPosition()) <= BATTLE_DISTANCE) {
				if(battleDialog == null){
					this.battleDialog = new ConfirmDialog((int) ((GameFrame.getResolution().getWidth() - 288) / 2), (int) ((GameFrame.getResolution().getHeight() - 256) / 2), Textures.getResource(BufferedImageProxy.class, "battle_dialog_background"));

					BACK_BUTTON.setEnabled(false);

					StarCharge.changeState(BaseState.States.PAUSE_STATE);
				}
			}
		}
	}
	// ============================================

	// GETTER AND SETTER
	// ============================================
	@Override
	public GUIElement[] getElements() {
		if (battleDialog != null) {
			return new GUIElement[] { BACK_BUTTON, battleDialog };
		} else {
			return new GUIElement[] { BACK_BUTTON };
		}
	}
	// ============================================
}
