package eu.eiting.StarCharge.visualization.menu.menuElements;

import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;
import java.awt.image.BufferedImage;

import eu.eiting.StarCharge.model.resources.datatypes.BufferedImageProxy;
import eu.eiting.StarCharge.visualization.utility.Color2D;

public class ButtonElement extends GUIElement {
	// OBJECT ATTRIBUTES
	// ============================================
	private final BufferedImageProxy INACTIVE;
	private final BufferedImageProxy ACTIVE;
	private final BufferedImageProxy DISABLED;
	private final BufferedImageProxy HOVERING;
	private boolean isActive					= false;
	private boolean isHovering					= false;
	// ============================================
	
	// CONSTRUCTOR
	// ============================================
	public ButtonElement(Rectangle dimension, BufferedImageProxy inactive, BufferedImageProxy active, BufferedImageProxy hovering, BufferedImageProxy disabled){
		super(dimension);
		
		this.INACTIVE	= inactive;
		this.ACTIVE		= active;
		this.DISABLED	= disabled;
		this.HOVERING	= hovering;
	}
	// ============================================
	
	// OBJECT METHODS
	// ============================================
	@Override
	protected boolean contains(Point point) {
		if(super.contains(point)){

			if(isActive){
				return !isTransparent(point, ACTIVE.getSource());
			} else if(isHovering){
				return !isTransparent(point, HOVERING.getSource());
			} else {
				return !isTransparent(point, INACTIVE.getSource());
			}
		}
		
		return false;
	}

	private boolean isTransparent(Point point, BufferedImage image){
		int relativeX = point.x - getDimension().x;
		int relativeY = point.y - getDimension().y;

		if (relativeX >= 0 && relativeY >= 0 && relativeX < image.getWidth() && relativeY < image.getHeight()) {
			if (Color2D.isTransparent(image.getRGB(relativeX, relativeY)) == false) {
				return false;
			} 
		}
		
		return true;
	}
	
	@Override
	public void onMousePress(MouseEvent event){
		if(contains(event.getPoint())){
			isActive = true;
		} else {
			isActive = false;
		}
	}
	
	@Override
	public void onMouseRelease(MouseEvent event) {
		isActive = false;
	}
	
	@Override
	public void onMouseMotion(MouseEvent event){
		if(contains(event.getPoint())){
			isHovering = true;
		} else {
			isHovering = false;
		}
	}
	// ============================================

	// GETTER AND SETTER
	// ============================================
	public BufferedImage getImage() {
		if(isEnabled()){
			if(isActive){
				return ACTIVE.getSource();
			} else if (isHovering) {
				return HOVERING.getSource();
			} else {
				return INACTIVE.getSource();
			}
		} else {
			return DISABLED.getSource();
		}
	}
	
	public boolean isActive() {
		return isActive;
	}
	
	public void setActive(boolean isActive) {
		this.isActive = isActive;
	}
	
	public boolean isHovering() {
		return isHovering;
	}
	
	public void setHovering(boolean isHovering) {
		this.isHovering = isHovering;
	}
	// ============================================
}