package eu.eiting.StarCharge.visualization.menu.menuElements;

import java.awt.Graphics;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;
import java.awt.image.BufferedImage;

import eu.eiting.StarCharge.model.resources.Textures;
import eu.eiting.StarCharge.model.resources.datatypes.BufferedImageProxy;
import eu.eiting.StarCharge.model.resources.datatypes.ImageList;
import eu.eiting.StarCharge.visualization.utility.ImageProcessing;

public class ConfirmDialog extends GUIElement {
	// CLASS ATTRIBUTES
	// ============================================
	public static final int NOTHING	= 0;
	public static final int CONFIRM	= 1;
	public static final int CANCEL	= 2;
	// ============================================
	
	// OBJECT ATTRIBUTES
	// ============================================
	private final ButtonElement CONFIRM_BUTTON;
	private final ButtonElement CANCEL_BUTTON;
	private int value								= NOTHING;
	private final BufferedImageProxy BACKGROUND; 
	// ============================================

	// CONSTRUCTOR
	// ============================================
	public ConfirmDialog(int x, int y, BufferedImageProxy background) {
		super(new Rectangle(x, y, 288, 256));
		
		this.BACKGROUND = background;
		
		ImageList images = Textures.getResource(ImageList.class, "engage_btn");
		this.CONFIRM_BUTTON			= new ButtonElement(new Rectangle(0, 0, 128, 64), images.get(0), images.get(1), images.get(2), images.get(3));
		this.CONFIRM_BUTTON.setParent(this);
		this.CONFIRM_BUTTON.setPosition(new Point(14, 178));
		
		images = Textures.getResource(ImageList.class, "retreat_btn");
		this.CANCEL_BUTTON		= new ButtonElement(new Rectangle(0, 0, 128, 64), images.get(0), images.get(1), images.get(2), images.get(3));
		this.CANCEL_BUTTON.setParent(this);
		this.CANCEL_BUTTON.setPosition(new Point(146, 178));
	}
	// ============================================
	
	// OBJECT METHODS
	// ============================================
	@Override
	public void onMousePress(MouseEvent event) {
		CONFIRM_BUTTON.onMousePress(event);
		CANCEL_BUTTON.onMousePress(event);
		
		if(CONFIRM_BUTTON.isActive()){
			value = CONFIRM;
		} else if(CANCEL_BUTTON.isActive()){
			value = CANCEL;
		} else {
			value = NOTHING;
		}
	}
	
	@Override
	public void onMouseRelease(MouseEvent event) {
		CONFIRM_BUTTON.onMouseRelease(event);
		CANCEL_BUTTON.onMouseRelease(event);
	}
	
	@Override
	public void onMouseMotion(MouseEvent event) {
		CONFIRM_BUTTON.onMouseMotion(event);
		CANCEL_BUTTON.onMouseMotion(event);
	}
	// ============================================
	
	// GETTER AND SETTER
	// ============================================
	public int getValue() {
		return value;
	}
	
	@Override
	public BufferedImage getImage() {
		BufferedImage product = ImageProcessing.createCopy(BACKGROUND.getSource());
		
		Graphics productG = product.getGraphics();
		
		productG.drawImage(CONFIRM_BUTTON.getImage(), CONFIRM_BUTTON.getPosition().x, CONFIRM_BUTTON.getPosition().y, null);
		productG.drawImage(CANCEL_BUTTON.getImage(), CANCEL_BUTTON.getPosition().x, CANCEL_BUTTON.getPosition().y, null);
		
		return product;
	}
	// ============================================
}
