package eu.eiting.StarCharge.visualization.menu.menuElements;

import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;
import java.util.ArrayList;

import eu.eiting.StarCharge.model.Drawable;

public abstract class GUIElement implements Drawable {
	// OBJECT ATTRIBUTES
	// ============================================
	private Rectangle dimension	= null;
	private boolean isEnabled	= true;
	private GUIElement parent	= null;
	private Point localPosition = null;
	private final ArrayList<GUIElement> CHILDREN = new ArrayList<GUIElement>();
	// ============================================
	
	// CONSTRUCTOR
	// ============================================
	public GUIElement(Rectangle dimension){
		this.dimension	= dimension;
	}
	// ============================================
	
	// OBJECT METHODS
	// ============================================
	protected boolean contains(Point point) {
		if (isEnabled) {
			return dimension.contains(point);
		}
		
		return false;
	}

	public abstract void onMousePress(MouseEvent event);

	public abstract void onMouseRelease(MouseEvent event);
	
	public abstract void onMouseMotion(MouseEvent event);
	// ============================================
	
	// GETTER AND SETTER
	// ============================================
	public Rectangle getDimension() {
		return dimension;
	}

	public void setPosition(Point point){
		if(parent == null){
			dimension = new Rectangle(point.x, point.y, dimension.width, dimension.height);
		} else {
			dimension = new Rectangle(parent.dimension.x + point.x, parent.dimension.y + point.y, dimension.width, dimension.height);
		
			localPosition = point;
		}
		
		for (GUIElement child : CHILDREN) {
			child.setPosition(child.getPosition());
		}
	}
	
	public void setEnabled(boolean value) {
		isEnabled = value;
	}

	public boolean isEnabled() {
		return isEnabled;
	}
	
	public void setParent(GUIElement parent) {
		if(this.parent != null){
			this.parent.CHILDREN.remove(this);
		}
		
		this.parent = parent;
		
		if(this.parent != null){
			this.parent.CHILDREN.add(this);
			
			localPosition = new Point(parent.getPosition().x - getDimension().x, parent.getPosition().y - getDimension().y);
		}
	}

	public Point getPosition() {
		if(parent != null){
			return localPosition;
		}
		
		return new Point(dimension.x, dimension.y);
	}
	
	public int getWidth() {
		return dimension.width;
	}
	
	public void setWidth(int width){
		dimension = new Rectangle(dimension.x, dimension.y, width, dimension.height);
	}
	
	public int getHeight() {
		return dimension.height;
	}
	
	public void setHeight(int height){
		dimension = new Rectangle(dimension.x, dimension.y, dimension.width, height);
	}
	// ============================================
}
