package eu.eiting.StarCharge.visualization.menu.menuElements;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;
import java.awt.image.BufferedImage;

public class StatusElement extends GUIElement {
	// OBJECT ATTRIBUTES
	// ============================================
	private final BarElement SHIELD_BAR;
	private final BarElement HULL_BAR;
	// ============================================
	
	// CONSTRUCTOR
	// ============================================
	public StatusElement(Rectangle dimension) {
		super(new Rectangle(dimension.x, dimension.y, dimension.width, 64));

		this.SHIELD_BAR	= new BarElement(new Rectangle(0, 0, getDimension().width - 23, 12), new Color(0, 156, 255, 128), new Color(20, 101, 152, 128), Color.WHITE);
		this.SHIELD_BAR.setParent(this);
		this.SHIELD_BAR.setPosition(new Point(12, 12));
		
		this.HULL_BAR	= new BarElement(new Rectangle(0, 0, getDimension().width - 23, 12), new Color(192, 255, 0, 128), Color.ORANGE, Color.WHITE);
		this.HULL_BAR.setParent(this);
		this.HULL_BAR.setPosition(new Point(12, 42));
	}
	// ============================================

	// OBJECT METHODS
	// ============================================
	@Override
	public void onMousePress(MouseEvent event) {
		SHIELD_BAR.onMousePress(event);
		HULL_BAR.onMousePress(event);
	}
	
	@Override
	public void onMouseRelease(MouseEvent event) {
		SHIELD_BAR.onMouseRelease(event);
		HULL_BAR.onMouseRelease(event);
	}
	
	@Override
	public void onMouseMotion(MouseEvent event) {
		SHIELD_BAR.onMouseMotion(event);
		HULL_BAR.onMouseMotion(event);
	}
	// ============================================
	
	// GETTER AND SETTER
	// ============================================
	@Override
	public BufferedImage getImage() {
		BufferedImage product = new BufferedImage(getDimension().width, getDimension().height, BufferedImage.TYPE_INT_ARGB);

		Graphics productG = product.getGraphics();

		productG.drawImage(SHIELD_BAR.getImage(), SHIELD_BAR.getPosition().x, SHIELD_BAR.getPosition().y, null);
		productG.drawImage(HULL_BAR.getImage(), HULL_BAR.getPosition().x, HULL_BAR.getPosition().y, null);

		productG.setColor(Color.WHITE);
		
		productG.drawLine(6, 6, getDimension().width - 6, 6);
		productG.drawLine(6, 6, 6, getDimension().height / 2 - 4);
		productG.drawLine(getDimension().width - 6, 6, getDimension().width - 6, getDimension().height / 2 - 4);
		
		productG.drawLine(6, getDimension().height / 2 + 4, getDimension().width - 6, getDimension().height / 2 + 4);
		productG.drawLine(6, getDimension().height / 2 + 4, 6, getDimension().height - 4);
		productG.drawLine(getDimension().width - 6, getDimension().height / 2 + 4, getDimension().width - 6, getDimension().height - 4);
		
		return product;
	}

	public BarElement getShieldBar(){
		return SHIELD_BAR;
	}
	
	public BarElement getHullBar() {
		return HULL_BAR;
	}
	// ============================================
}

