package eu.eiting.StarCharge.visualization.menu.menuElements;

import java.awt.Graphics;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;
import java.awt.image.BufferedImage;
import java.util.ArrayList;

import eu.eiting.StarCharge.control.StarCharge;
import eu.eiting.StarCharge.model.GameModel;
import eu.eiting.StarCharge.model.Updateable;
import eu.eiting.StarCharge.model.listener.ChangeListener;
import eu.eiting.StarCharge.model.resources.Textures;
import eu.eiting.StarCharge.model.resources.datatypes.BufferedImageProxy;
import eu.eiting.StarCharge.model.resources.datatypes.ImageList;
import eu.eiting.StarCharge.visualization.GameFrame;


public class WeaponElement extends GUIElement implements Updateable, ChangeListener<GameModel> {
	// OBJECT ATTRIBUTES
	// ====================================
	private final ArrayList<SwitchElement> ELEMENTS	= new ArrayList<>();
	private final BufferedImageProxy IMAGE;
	private GameModel gameModel						= null;
	private int selectedSlot						= 0;
	private SwitchGroup group						= null;
	// ====================================
	
	// CONSTRUCTOR
	// ====================================
	public WeaponElement(Rectangle dimension) {
		super(dimension);
		
		this.IMAGE	= Textures.getResource(BufferedImageProxy.class, "weapon_slot_background");

		StarCharge.addGameModelListener(this);
	}
	// ====================================

	// OBJECT METHODS
	// ====================================
	private void setupElements(){
		ELEMENTS.clear();
		
		int width = gameModel.player.getStarShip().WEAPONS.getSlotCount() * 64;

		setWidth(width);
		
		setPosition(new Point(GameFrame.getResolution().width - width, getDimension().y));
		
		group = new SwitchGroup();

		for (int index = 0; index < gameModel.player.getStarShip().WEAPONS.getSlotCount(); index++) {
			ImageList icon = Textures.getResource(ImageList.class, gameModel.player.getStarShip().WEAPONS.getSlot(index).getWeaponType() + "_icon");
			
			SwitchElement slot = new SwitchElement(new Rectangle(0, 0, 50, 50), icon.get(0), icon.get(1), icon.get(2), icon.get(3));
			slot.setParent(this);
			slot.setPosition(new Point(7 + index * 64, 7));
			slot.setGroup(group);
			
			if (gameModel.player.getStarShip().WEAPONS.getSlot(index).getEnabled() == false) {
				slot.setEnabled(false);
			}
			
			ELEMENTS.add(slot);
		}
	}

	@Override
	public void update() {
		boolean isSearching = true;
		
		for(int index = 0; index < ELEMENTS.size() && isSearching; index++){
			if(selectedSlot != index && ELEMENTS.get(index).isActive()){
				selectedSlot = index;
				
				for (int i = 0; i < gameModel.player.getStarShip().WEAPONS.getSlotCount(); i++) {
					gameModel.player.getStarShip().WEAPONS.getSlot(i).setActive(false);
				}
				
				gameModel.player.getStarShip().WEAPONS.getSlot(selectedSlot).setActive(true);
				
				isSearching = false;
			}
		}

		for (int index = 0; index < gameModel.player.getStarShip().WEAPONS.getSlotCount(); index++) {
			if(gameModel.player.getStarShip().WEAPONS.getSlot(index).isActive() && index != selectedSlot){
				group.setActive(ELEMENTS.get(index));
			}
		}
		
		if (ELEMENTS.size() == gameModel.player.getStarShip().WEAPONS.getSlotCount()) {
			for (int i = 0; i < ELEMENTS.size(); i++) {
				if (gameModel.player.getStarShip().WEAPONS.getSlot(i).getEnabled()) {
					ELEMENTS.get(i).setEnabled(true);
				} else {
					ELEMENTS.get(i).setEnabled(false);
				}
			}
		} else {
			setupElements();
		}
	}
	
	@Override
	public void onChange(GameModel model) {
		gameModel = model;
	}
	

	@Override
	public void onMousePress(MouseEvent event) {
		for (GUIElement element : ELEMENTS) {
			element.onMousePress(event);
		}
	}
	
	@Override
	public void onMouseRelease(MouseEvent event) {
		for (GUIElement element : ELEMENTS) {
			element.onMouseRelease(event);
		}
	}
	
	@Override
	public void onMouseMotion(MouseEvent event) {
		for (GUIElement element : ELEMENTS) {
			element.onMouseMotion(event);
		}
	}
	// ====================================
	
	// GETTER AND SETTER
	// ====================================
	@Override
	public BufferedImage getImage() {
		BufferedImage product = new BufferedImage(getDimension().width, getDimension().height, BufferedImage.TYPE_INT_ARGB);
		
		Graphics productG = product.getGraphics();

		for(int index = 0; index < ELEMENTS.size(); index++) {
			productG.drawImage(IMAGE.getSource(), index * 64, 0, null);
			productG.drawImage(ELEMENTS.get(index).getImage(), ELEMENTS.get(index).getPosition().x, ELEMENTS.get(index).getPosition().y, null);
		}

		return product;
	}
	// ====================================
}
