package eu.eiting.StarCharge.visualization.menu.menuElements.list;

import java.awt.Graphics;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;
import java.awt.image.BufferedImage;
import java.util.ArrayList;

import eu.eiting.StarCharge.model.resources.Textures;
import eu.eiting.StarCharge.model.resources.datatypes.BufferedImageProxy;
import eu.eiting.StarCharge.model.resources.datatypes.ImageList;
import eu.eiting.StarCharge.visualization.menu.menuElements.ButtonElement;
import eu.eiting.StarCharge.visualization.menu.menuElements.GUIElement;
import eu.eiting.StarCharge.visualization.menu.menuElements.SwitchGroup;

public class ListElement extends GUIElement {
	// CLASS ATTRIBUTES
	//====================================
	public static final int NO_BUTTON		= 0;
	public static final int OK_BUTTON		= 1;
	public static final int DELETE_BUTTON	= 2;
	public static final int ADD_BUTTON		= 3;
	//====================================

	// OBJECT ATTRIBUTES
	// ====================================
	private final BufferedImageProxy TOP_ELEMENT;
	private final BufferedImageProxy ENTRY_ELEMENT;
	private final BufferedImageProxy BOTTOM_ELEMENT;
	private ButtonElement addButton;
	private ButtonElement deleteButton;
	private ButtonElement confirmButton;
	private final ArrayList<ListEntry> ENTRIES		= new ArrayList<>();
	private ListEntry selected						= null;
	private String[] elements						= null;
	private int value								= NO_BUTTON;
	// ====================================

	// CONSTRUCTOR
	// ====================================
	public ListElement(int x, int y) {
		super(new Rectangle(x, y, 256, 256));
		
		ImageList images = Textures.getResource(ImageList.class, "profile_element");

		this.TOP_ELEMENT = images.get(0);
		this.ENTRY_ELEMENT = images.get(1);
		this.BOTTOM_ELEMENT = images.get(2);

		this.elements = new String[0];

		setup();
	}
	// ====================================

	// OBJECT METHODS
	// ====================================
	private void setup() {
		SwitchGroup group = new SwitchGroup();
		
		int currentHeight = 64;
		
		ENTRIES.clear();

		for (int index = 0; index < elements.length; index++) {
			ListEntry entry = new ListEntry(new Rectangle(0, 0, getDimension().width, 64), ENTRY_ELEMENT, elements[index], index, group);
			entry.setParent(this);
			entry.setPosition(new Point(0, currentHeight));
			ENTRIES.add(entry);
			
			currentHeight += 64;
		}
		
		setHeight(currentHeight + 64);

		ImageList images = Textures.getResource(ImageList.class, "add_btn");
		this.addButton = new ButtonElement(new Rectangle(0, 0, 64, 32), images.get(0), images.get(1), images.get(2), images.get(3));
		this.addButton.setParent(this);
		this.addButton.setPosition(new Point(63, currentHeight + 16));
		
		images = Textures.getResource(ImageList.class, "del_btn");
		this.deleteButton = new ButtonElement(new Rectangle(0, 0, 64, 32), images.get(0), images.get(1), images.get(2), images.get(3));
		this.deleteButton.setParent(this);
		this.deleteButton.setPosition(new Point(129, currentHeight + 16));
		this.deleteButton.setEnabled(false);
		
		images = Textures.getResource(ImageList.class, "ok_btn");
		this.confirmButton = new ButtonElement(new Rectangle(0, 0, 32, 32), images.get(0), images.get(1), images.get(2), images.get(3));
		this.confirmButton.setParent(this);
		this.confirmButton.setPosition(new Point(195, currentHeight + 16));
		this.confirmButton.setEnabled(false);
		
		validateSelection();
	}
	
	private void validateSelection(){
		selected = null;

		for (int index = 0; index < ENTRIES.size(); index++) {
			if (ENTRIES.get(index).isActive()) {
				selected = ENTRIES.get(index);
				
				this.deleteButton.setEnabled(true);
				this.confirmButton.setEnabled(true);
			}
		}
	}

	@Override
	public void onMousePress(MouseEvent event) {
		addButton.onMousePress(event);
		deleteButton.onMousePress(event);
		confirmButton.onMousePress(event);

		for (ListEntry element : ENTRIES) {
			element.onMousePress(event);
		}

		validateSelection();

		value = NO_BUTTON;
		
		if (addButton.isActive()) {
			value = ADD_BUTTON;
		}

		if (deleteButton.isActive()) {
			value = DELETE_BUTTON;
		}

		if (confirmButton.isActive()) {
			value = OK_BUTTON;
		}
	}
	
	@Override
	public void onMouseRelease(MouseEvent event) {
		addButton.onMouseRelease(event);
		deleteButton.onMouseRelease(event);
		confirmButton.onMouseRelease(event);

		for (ListEntry element : ENTRIES) {
			element.onMouseRelease(event);
		}
	}

	@Override
	public void onMouseMotion(MouseEvent event) {
		addButton.onMouseMotion(event);
		deleteButton.onMouseMotion(event);
		confirmButton.onMouseMotion(event);

		for (ListEntry element : ENTRIES) {
			element.onMouseMotion(event);
		}
	}
	// ====================================

	// GETTER AND SETTER
	// ====================================
	public void setElements(String[] elements) {
		this.elements = elements;

		setup();
	}
	
	public int getValue() {
		return value;
	}
	
	public ListEntry getSelected() {
		return selected;
	}

	@Override
	public BufferedImage getImage() {
		BufferedImage product = new BufferedImage(getDimension().width, getDimension().height, BufferedImage.TYPE_INT_ARGB);
		
		Graphics productG = product.getGraphics();

		productG.drawImage(TOP_ELEMENT.getSource(), 0, 0, null);

		for (ListEntry entry : ENTRIES) {
			productG.drawImage(entry.getImage(), entry.getPosition().x, entry.getPosition().y, null);
		}

		productG.drawImage(BOTTOM_ELEMENT.getSource(), 0, getDimension().height - BOTTOM_ELEMENT.getSource().getHeight(), null);

		productG.drawImage(addButton.getImage(), addButton.getPosition().x, addButton.getPosition().y, null);
		productG.drawImage(deleteButton.getImage(), deleteButton.getPosition().x, deleteButton.getPosition().y, null);
		productG.drawImage(confirmButton.getImage(), confirmButton.getPosition().x, confirmButton.getPosition().y, null);

		return product;
	}
	// ====================================
}
