package eu.eiting.StarCharge.visualization.menu.menuElements.list;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;
import java.awt.image.BufferedImage;

import eu.eiting.StarCharge.model.resources.Textures;
import eu.eiting.StarCharge.model.resources.datatypes.BufferedImageProxy;
import eu.eiting.StarCharge.model.resources.datatypes.ImageList;
import eu.eiting.StarCharge.visualization.menu.menuElements.GUIElement;
import eu.eiting.StarCharge.visualization.menu.menuElements.SwitchElement;
import eu.eiting.StarCharge.visualization.menu.menuElements.SwitchGroup;

public class ListEntry extends GUIElement{
	// CLASS ATTRIBUTES
	// ====================================
	private static final Color ACTIVE			= new Color(0, 186, 255);
	private static final Color HOVERING			= Color.RED;
	private static final Color INACTIVE			= new Color(142, 0, 0);
	private static final Font FONT				= new Font("Downlink", Font.PLAIN, 16);
	// ====================================	
	
	// OBJECT ATTRIBUTES
	// ====================================
	protected final String VALUE;
	protected final SwitchElement BUTTON;
	private final BufferedImageProxy IMAGE;
	private final int INDEX;
	// ====================================

	// CONSTRUCTOR
	// ====================================
	public ListEntry(Rectangle dimension, BufferedImageProxy image, String value, int index, SwitchGroup group) {
		super(dimension);
		
		this.IMAGE		= image;
		this.VALUE		= value;
		this.INDEX		= index;
		
		ImageList images = Textures.getResource(ImageList.class, "profile_element_btn");
		
		this.BUTTON = new SwitchElement(new Rectangle(0, 0, 174, 48), images.get(0), images.get(1), images.get(2), images.get(3));
		this.BUTTON.setParent(this);
		this.BUTTON.setPosition(new Point(41, 8));
		
		this.BUTTON.setGroup(group);
	}
	// ====================================
	
	// OBJECT METHODS
	// ====================================
	@Override
	public void onMousePress(MouseEvent event) {
		BUTTON.onMousePress(event);
	}

	@Override
	public void onMouseRelease(MouseEvent event) { }

	@Override
	public void onMouseMotion(MouseEvent event) {
		BUTTON.onMouseMotion(event);
	}
	// ====================================
	
	// GETTER AND SETTER
	// ====================================
	@Override
	public BufferedImage getImage() {
		BufferedImage product = new BufferedImage(getDimension().width, getDimension().height, BufferedImage.TYPE_INT_ARGB);
		
		Graphics productG = product.getGraphics();
		
		productG.drawImage(IMAGE.getSource(), 0, 0, null);
		productG.drawImage(BUTTON.getImage(), BUTTON.getPosition().x, BUTTON.getPosition().y, null);

		if(BUTTON.isActive()){
			productG.setColor(ACTIVE);
		} else if(BUTTON.isHovering()){
			productG.setColor(HOVERING);
		} else {
			productG.setColor(INACTIVE);
		}
		
		productG.setFont(FONT);
		productG.drawString(VALUE, 48, 28);
		
		return product;
	}
	
	public boolean isActive() {
		return BUTTON.isActive();
	}
	
	public int getIndex() {
		return INDEX;
	}
	// ====================================
}
