package eu.eiting.StarCharge.visualization.menu.menuElements.solarSystem;

import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;
import java.awt.image.BufferedImage;

import eu.eiting.StarCharge.model.Updateable;
import eu.eiting.StarCharge.model.resources.datatypes.BufferedImageProxy;
import eu.eiting.StarCharge.model.solarSystem.Star;
import eu.eiting.StarCharge.visualization.menu.menuElements.GUIElement;
import eu.eiting.StarCharge.visualization.utility.Color2D;

public class PlanetElement extends GUIElement implements Updateable {
	// OBJECT ATTRIBUTES
	// ====================================
	private final Star PARENT;
	private final BufferedImageProxy INACTIVE;
	private final BufferedImageProxy HOVERING;
	private boolean isHovering					= false;
	// ====================================

	// CONSTRUCTOR
	// ====================================
	public PlanetElement(Rectangle dimension, BufferedImageProxy inactive, BufferedImageProxy hovering, Star parent) {
		super(dimension);
		this.PARENT		= parent;
		this.INACTIVE	= inactive;
		this.HOVERING	= hovering;
	}
	// ====================================

	// OBJECT METHODS
	// ====================================
	@Override
	public void update() {
		setPosition(new Point((int) (PARENT.getTransform().getPosition().x - getDimension().width / 2), (int) (PARENT.getTransform().getPosition().y - getDimension().height / 2)));
	}

	@Override
	public void onMousePress(MouseEvent event) {}

	@Override
	public void onMouseRelease(MouseEvent event) {}

	@Override
	public void onMouseMotion(MouseEvent event) {
		if(contains(event.getPoint())){
			if(!isHovering && isTransparent(event.getPoint(), INACTIVE.getSource())){
				isHovering = true;
			} else if(isHovering && !isTransparent(event.getPoint(), HOVERING.getSource())){
				isHovering = false;
			}
		} else {
			isHovering = false;
		}
	}
	
	private boolean isTransparent(Point point, BufferedImage image){
		int relativeX = point.x - getDimension().x;
		int relativeY = point.y - getDimension().y;

		if (relativeX >= 0 && relativeY >= 0 && relativeX < image.getWidth() && relativeY < image.getHeight()) {
			if (Color2D.isTransparent(image.getRGB(relativeX, relativeY)) == false) {
				return true;
			} 
		}
		
		return false;
	}
	// ====================================

	// GETTER AND SETTER
	// ====================================
	@Override
	public BufferedImage getImage() {
		if (isHovering) {
			return HOVERING.getSource();
		}
		
		return INACTIVE.getSource();
	}
	// ====================================
}
