package eu.eiting.StarCharge.visualization.views;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.util.List;

import eu.eiting.StarCharge.control.controller.BaseController;
import eu.eiting.StarCharge.control.controller.GameController;
import eu.eiting.StarCharge.model.Screen;
import eu.eiting.StarCharge.model.objects.Projectile;
import eu.eiting.StarCharge.model.objects.starship.StarShip;
import eu.eiting.StarCharge.model.resources.ImageTool;
import eu.eiting.StarCharge.model.solarSystem.Fleet;
import eu.eiting.StarCharge.model.structure.Vector2;
import eu.eiting.StarCharge.visualization.menu.GameGUI;
import eu.eiting.StarCharge.visualization.menu.GameMenuGUI;
import eu.eiting.StarCharge.visualization.menu.container.GUIContainer;
import eu.eiting.StarCharge.visualization.menu.menuElements.GUIElement;
import eu.eiting.StarCharge.visualization.utility.Point2D;

public class GameView extends BaseView implements Screen{
	// CLASS ATTRIBUTES
	// ====================================
	private static final long serialVersionUID		= 5822743569702743513L;
	private static Screen screen					= null;
	// ====================================

	// OBJECT ATTRIBUTES
	// ====================================
	private BufferedImage representation			= null;
	private Point offset							= Point2D.ZERO;
	private ArrayList<BaseController> controller	= null;
	private final GUIContainer gui					= new GUIContainer();
	// ====================================

	// TODO: remove temporary attributes
	// ====================================
	BufferedImage background = null;
	// ====================================

	// CONSTRUCTOR
	// ====================================
	public GameView(Dimension currentResolution, String name) {
		super(currentResolution);
		// set up the view
		// ====================================
		setSize(currentResolution);
		// ====================================

		// initialize or set up object attributes
		// ====================================
		this.controller								= new ArrayList<BaseController>();
		this.representation							= new BufferedImage(getWidth(), getHeight(), BufferedImage.TYPE_INT_ARGB);
		// ====================================

		// add the GameController
		// ====================================
		// add the GameController to the controller list
		controller.add(new GameController(gui));

		addKeyListener(controller.get(0));
		addMouseListener(controller.get(0));
		addMouseMotionListener(controller.get(0));
		// ====================================

		// set up the MenuContainer
		// =====================================
		this.gui.addMenu(GUIContainer.Menus.GAME_HUD, new GameGUI(gui));
		this.gui.currentMenu.setActive(true);
		this.gui.addMenu(GUIContainer.Menus.GAME_MENU, new GameMenuGUI(gui));
		// =====================================

		background = ImageTool.load("space.png");
		
		screen = this;
	}

	// ====================================

	// CLASS METHODS
	// ====================================
	@Override
	public Vector2 screenToWorld(Point p) {
		if(getGameModel() != null){
			return new Vector2(p.x - offset.x, p.y - offset.y);
		} else {
			return Vector2.ZERO;
		}
	}

	@Override
	public Point worldToScreen(Vector2 v) {
		if(getGameModel() != null){
			return new Point((int) (v.x + offset.x), (int) (v.y + offset.y));
		} else {
			return Point2D.ZERO;
		}
	}

	public static Screen getScreen() {
		return screen;
	}
	// ====================================

	// OBJECT METHODS
	// ====================================
	private void calculateOffset() {
		if(getGameModel() != null && getGameModel().player != null && getGameModel().player.getStarShip() != null){
			offset = new Point((int) (getWidth() / 2 - getGameModel().player.getStarShip().getTransform().getPosition().x), (int) (getHeight() / 2 - getGameModel().player.getStarShip().getTransform().getPosition().y));
		}
	}

	@Override
	protected void paintComponent(Graphics g) {
		if (getGameModel() != null) {
			// recalculate the offset of the camera
			// ====================================
			calculateOffset();
			// ====================================

			// prepare the representation
			// ====================================
			Graphics2D g2D = (Graphics2D) representation.getGraphics();
			// ====================================

			// call the paint methods for the different tasks
			// ====================================
			paintBackground(g2D);
			paintShips(g2D);
			paintProjectiles(g2D);
			paintInterface(g2D);
			// ====================================

			// draw the representation to the screen
			// ====================================
			g.drawImage(representation, 0, 0, null);
			// ====================================
		}
	}

	private void paintBackground(Graphics g) {
		int yElements = (int) (Math.ceil(getHeight() / background.getHeight()) + 2);
		int xElements = (int) (Math.ceil(getWidth() / background.getWidth()) + 2);

		int localXOffset = offset.x % background.getWidth();
		int localYOffset = offset.y % background.getHeight();

		for (int y = -1; y < yElements; y++) {
			for (int x = -1; x < xElements; x++) {
				g.drawImage(background, x * background.getWidth() + localXOffset, y * background.getHeight() + localYOffset, null);
			}
		}
	}

	private void paintShip(Graphics g, StarShip ship) {
		paintImage(g, ship.getImage(), (int) ship.getTransform().getPosition().x, (int) ship.getTransform().getPosition().y);
	}

	private void paintShips(Graphics g) {
		Fleet ships = getGameModel().getBattleField().getAggressor();

		for (int i = 0; i < ships.getStarShipCount(); i++) {
			paintShip(g, ships.getStarShip(i));
		}

		ships = getGameModel().getBattleField().getDefender();

		for (int i = 0; i < ships.getStarShipCount(); i++) {
			paintShip(g, ships.getStarShip(i));
		}
	}

	private void paintProjectiles(Graphics g) {
		List<Projectile> projetiles = getGameModel().getBattleField().getProjectiles();

		for (int i = 0; i < projetiles.size(); i++) {
			paintImage(g, projetiles.get(i).getImage(), (int) projetiles.get(i).getTransform().getPosition().x, (int) projetiles.get(i).getTransform().getPosition().y);
		}
	}

	private void paintInterface(Graphics g) {
		if (gui.currentMenu.isActive()) {
			for (GUIElement me : gui.getGuiElements()) {
				g.drawImage(me.getImage(), me.getPosition().x,
						me.getPosition().y, null);
			}
		}
	}

	private void paintImage(Graphics g, BufferedImage image, int x, int y) {
		g.drawImage(image, x - image.getWidth() / 2 + offset.x, y - image.getHeight() / 2 + offset.y, null);
	}
	// ====================================

	// GETTER AND SETTER
	// ====================================
	@Override
	public BaseController[] getController() {
		return controller.toArray(new BaseController[controller.size()]);
	}
	// ====================================
}
