package eu.eiting.StarCharge.visualization.views;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.image.BufferedImage;
import java.util.List;

import eu.eiting.StarCharge.control.controller.BaseController;
import eu.eiting.StarCharge.control.controller.SolarSystemController;
import eu.eiting.StarCharge.model.solarSystem.Planet;
import eu.eiting.StarCharge.model.structure.Vector2;
import eu.eiting.StarCharge.visualization.menu.SolarSystemGUI;
import eu.eiting.StarCharge.visualization.menu.SolarSystemMenuGUI;
import eu.eiting.StarCharge.visualization.menu.container.GUIContainer;
import eu.eiting.StarCharge.visualization.menu.menuElements.GUIElement;
import eu.eiting.StarCharge.visualization.utility.Point2D;

public class SolarSystemView extends BaseView{
	// CLASS ATTRIBUTES
	// ====================================
	private static final long serialVersionUID 	= -2932097798655049549L;
	// ====================================

	// OBJECT ATTRIBUTES
	// ====================================
	private Vector2 camera						= null;
	private BufferedImage representation		= null;
	private Point offset						= Point2D.ZERO;
	private GUIContainer gui					= null;
	private BaseController controller			= null;
	private SolarSystemGUI baseView = null;
	// ====================================
	
	// CONSTRUCTOR
	// ====================================
	public SolarSystemView(Dimension currentResolution) {
		super(currentResolution);
		// initialize view
		// ====================================
		this.camera			= Vector2.ZERO;
		this.representation = new BufferedImage(getWidth(), getHeight(), BufferedImage.TYPE_INT_ARGB);
		// ====================================
		
		// set up the MenuContainer
		//=====================================
		this.gui = new GUIContainer();
		this.gui.addMenu(GUIContainer.Menus.SOLAR_SYSTEM_HUD, new SolarSystemMenuGUI(gui));
		//=====================================

		baseView = new SolarSystemGUI(gui);
		
		// set up the controller
		// ====================================
		this.controller = new SolarSystemController(gui, baseView);
		this.addMouseListener(controller);
		this.addMouseMotionListener(controller);
		// ====================================
		
	}
	// ====================================

	// OBJECT METHODS
	// ====================================
	@Override
	protected void paintComponent(Graphics g) {
		// prepare representation
		// ====================================
		Graphics representationGraphics = representation.getGraphics();
		
		representationGraphics.setColor(Color.BLACK);
		
		representationGraphics.fillRect(0, 0, representation.getWidth(), representation.getHeight());
		// ====================================
		
		baseView.update();
		
		calculateOffset();
		
		// paint parts of the view
		// ====================================
		paintOrbits(representationGraphics);
		paintInterface(representationGraphics);
		// ====================================

		g.drawImage(representation, 0, 0, null);
	}
	
	private void calculateOffset(){
		offset = new Point((int) (getWidth() / 2 - camera.x), (int) (getHeight() / 2 - camera.y));
	}

	public void paintOrbits(Graphics g){
		if(getGameModel() != null && getGameModel().solarSystem != null){
			List<Planet> planets = getGameModel().solarSystem.getPlanets();

			g.setColor(new Color(255, 255, 255, 128));

			for (int i = 0; i < planets.size(); i++) {
			
				int diameter = (int) (planets.get(i).orbitDistance * 2);
				int y = (int) (getGameModel().solarSystem.getSun().getTransform().getPosition().x - planets.get(i).orbitDistance);
				int x = (int) (getGameModel().solarSystem.getSun().getTransform().getPosition().y - planets.get(i).orbitDistance);
			
				g.drawOval(x + offset.x, y + offset.y, diameter, diameter);
			}
		}
	}
	
	private void paintInterface(Graphics g){
		for (GUIElement element : baseView.getElements()) {
			g.drawImage(element.getImage(), element.getPosition().x + offset.x, element.getPosition().y + offset.y, null);
		}
		
		for (GUIElement element : gui.currentMenu.getElements()) {
			g.drawImage(element.getImage(), element.getPosition().x, element.getPosition().y, null);
		}
	}
	// ====================================

	// GETTER AND SETTER
	// ====================================
	@Override
	public BaseController[] getController() {
		return new BaseController[]{controller};
	}
	// ====================================
}
